(function(factory) {
  if (typeof exports === 'object' && typeof module === 'object') {
    module.exports = factory();
  } else {
    factory();
  }
})(function() {
  try {
    var webinsight = {};

        /**
         * 获取 [min, max] 中的值
         * @param {number} v
         * @param {number} min
         * @param {number} max
         */
        const getInsideValue = (v, min, max) => {
          const _v = v;
          v = Math.max(v, min);
          v = Math.min(v, max);
          console.log(_v, min, max);
          return v;
        };

        /**
         * 将对象中的数值属性四舍五入
         * @template T extends object
         * @param {T} obj
         */
        const getRounded = (obj) => {
          if (obj == null) return obj;

          const data = Object.assign({}, obj);
          for (const k in obj) {
            const v = obj[k];
            // @ts-ignore
            obj[k] = typeof v === "number" ? Math.round(v) : v;
          }
          return data;
        };

        /**
         * 获取目标位置与限制范围的重叠区域尺寸，无则返回 null
         * @param {Record<string, number>} option
         * @example
         * getInsideRect({
         *   // 目标位置
         *   top: 0,
         *   bottom: 100,
         *   left: 10,
         *   right: 50,
         *
         *   // 限制范围
         *   yMin: 0,
         *   yMax: 300,
         *   xMin: 20,
         *   xMax: 100
         * })
         */
        const getInsideRect = (option) => {
          const { top, bottom, left, right, yMin, yMax, xMin, xMax } = option;

          // 1 在外面
          const topOutside = bottom <= yMin;
          if (topOutside) return null;
          const bottomOutside = top >= yMax;
          if (bottomOutside) return null;
          const leftOutside = right <= yMin;
          if (leftOutside) return null;
          const rightOutside = left >= xMax;
          if (rightOutside) return null;

          // 2 大于可视区
          const containViewport =
            top <= yMin && bottom >= yMax && left <= xMin && right >= xMax;
          if (containViewport) {
            const data = {
              top: yMin,
              bottom: yMax,
              left: xMin,
              right: xMax,
              width: 0,
              height: 0,
            };
            data.width = data.right - data.left;
            data.height = data.bottom - data.top;
            return data;
          }

          // 3 可视区包裹元素，或存在重叠区域
          const data = {
            top: getInsideValue(top, 0, yMax),
            bottom: getInsideValue(bottom, 0, yMax),
            left: getInsideValue(left, 0, xMax),
            right: getInsideValue(right, 0, xMax),
            width: 0,
            height: 0,
          };
          data.width = data.right - data.left;
          data.height = data.bottom - data.top;
          return data;
        };

        /**
         * 获取元素与可视区重叠区域位置，无则返回 null
         * @param {HTMLElement} el
         */
        const getElementInsideRect = (el) => {
          // 1 不是元素
          const isElement = el instanceof HTMLElement;
          if (!isElement) return null;

          // 2 元素和可视区尺寸
          const rect = el.getBoundingClientRect();
          const { top, bottom, left, right } = rect;
          const { innerWidth: xMax, innerHeight: yMax } = window;
          const xMin = 0;
          const yMin = 0;
          const data = getRounded(
            getInsideRect({
              top,
              bottom,
              left,
              right,
              yMin,
              yMax,
              xMin,
              xMax,
            })
          );
          if (!data) return null;

          // 6 绘制遮挡元素
          // const div = document.createElement("div");
          // const px = "px";
          // /** @type {Partial<CSSStyleDeclaration>} */
          // const style = {
          //   position: "fixed",
          //   width: data.width + px,
          //   height: data.height + px,
          //   top: data.top + px,
          //   left: data.left + px,
          //   background: "red",
          //   zIndex: "10",
          // };
          // Object.assign(div.style, style);
          // document.body.append(div);

          return data;
        };

        /**
         * 获取可视区中的 input，并返回其坐标位置
         */
        const useInputObserver = (selectors = ["input"]) => {
          /** @type {(() => void) | null} */
          let cleaner = null;
          /** @type {MutationObserver | null} */
          let observer = null;

          /**
           * 获取所有选择器匹配的元素
           */
          const getPrivateElList = () => {
            /** @type {HTMLElement[]} */
            const elList = [];
            selectors.forEach((sel) => {
              try {
                /** @type {HTMLElement[]} */
                const list = Array.from(document.querySelectorAll(sel));
                elList.push(...list);
              } catch (e) {
                // console.error('', e);
              }
            });
            return elList;
          };

          /**
           * 获取所有可见元素的可见坐标
           * @param {HTMLElement[]} elList
           */
          const getVisibleRectList = (elList) => {
            /** @type {{ top: number; bottom: number; left: number; right: number; width: number; height: number; }[]} */
            const rectList = [];

            elList.forEach((el) => {
              const rect = getElementInsideRect(el);
              if (rect == null) return;
              rectList.push(rect);
            });

            return rectList;
          };

          /**
           * @todo 抛出数据到原生应用中
           */
          const onUpdate = () => {
            const elList = getPrivateElList();
            const rectList = getVisibleRectList(elList);
            console.log('send content:' + JSON.stringify(rectList))
            AppInsightJSBridge.sensitiveView(JSON.stringify(rectList))
/*            window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage(
              JSON.stringify({ event: "sensitive", data: rectList })
            );*/
          };

          const onUpdateThrottled = (() => {
            const delay = 200;
            let last = 0;
            let timer = 0;
            return () => {
              // 节流，首次也触发
              const now = Date.now();
              const canEmit = now - last >= delay;
              if (canEmit) {
                onUpdate();
                last = now;
                return;
              }

              // 防抖
              window.clearTimeout(timer);
              timer = window.setTimeout(() => {
                onUpdate();
                last = Date.now();
              }, delay);
            };
          })();

          const start = () => {
            stop();

            setTimeout(() => {
              onUpdate();
            }, 100);

            window.addEventListener("scroll", onUpdateThrottled);
            observer = new MutationObserver(onUpdateThrottled);
            observer.observe(document.body, {
              subtree: true,
              childList: true,
              attributes: true,
            });

            cleaner = () => {
              window.removeEventListener("scroll", onUpdateThrottled);
              observer.disconnect();
              observer = null;
            };
          };

          const stop = () => {
            if (typeof cleaner === "function") cleaner();
          };

          const addSensitiveView = (selector) => {
            selectors.push(selector);
          };

          return { start, stop, addSensitiveView };
        };

        const input = useInputObserver(["input"]);
        setTimeout(() => {
          input.start();
        }, 100);

        var webinsight = {};
        webinsight.addSensitiveView = input.addSensitiveView;

    let para = {
      name: 'shujiantec',
      app_js_bridge : true,
      heatmap: {
        clickmap:'default',
        scroll_notice_map:'default',
        collect_tags:{
          div : true
        }
      }
    };
    var n = para.name, w = window;
    if(typeof(w['webInsightDataAnalytic']) !== 'undefined') {

    } else {
      w['webInsightDataAnalytic'] = para.name;
      w[n] = w[n] || function(a) {return function() {(w[n]._q = w[n]._q || []).push([a, arguments]);}};
      var ifs = ['track','quick','register','registerPage','registerOnce','trackLink','clearAllRegister','getAppStatus'];
        for (var i = 0; i < ifs.length; i++) {
            w[n][ifs[i]] = w[n].call(null, ifs[i]);
        }

        if (!w[n]._t) {
            w[n].para = para;
        }
    }

    window.webinsight = webinsight;
    webinsight.modules = {};

    var _ = (webinsight._ = {});

    (function() {
      var objectTypes = {
        "function": true,
        "object": true
      };
      var root = objectTypes[typeof window] && window || this;

      function runInContext(context, exports) {
        context || (context = root.Object());
        exports || (exports = root.Object());

        var Number = context.Number || root.Number,
          String = context.String || root.String,
          Object = context.Object || root.Object,
          Date = context.Date || root.Date,
          SyntaxError = context.SyntaxError || root.SyntaxError,
          TypeError = context.TypeError || root.TypeError,
          Math = context.Math || root.Math,
          nativeJSON = context.JSON || root.JSON;

        if (typeof nativeJSON == "object" && nativeJSON) {
          exports.stringify = nativeJSON.stringify;
          exports.parse = nativeJSON.parse;
        }

        var objectProto = Object.prototype,
          getClass = objectProto.toString,
          isProperty = objectProto.hasOwnProperty,
          undefined;

        function attempt(func, errorFunc) {
          try {
            func();
          } catch (exception) {
            if (errorFunc) {
              errorFunc();
            }
          }
        }

        var isExtended = new Date(-3509827334573292);
        attempt(function() {
          isExtended = isExtended.getUTCFullYear() == -109252 && isExtended.getUTCMonth() === 0 && isExtended.getUTCDate() === 1 &&
            isExtended.getUTCHours() == 10 && isExtended.getUTCMinutes() == 37 && isExtended.getUTCSeconds() == 6 && isExtended.getUTCMilliseconds() == 708;
        });

        function has(name) {
          if (has[name] != null) {
            return has[name];
          }
          var isSupported;
          if (name == "bug-string-char-index") {
            isSupported = "a" [0] != "a";
          } else if (name == "json") {
            isSupported = has("json-stringify") && has("date-serialization") && has("json-parse");
          } else if (name == "date-serialization") {
            isSupported = has("json-stringify") && isExtended;
            if (isSupported) {
              var stringify = exports.stringify;
              attempt(function() {
                isSupported =
                  stringify(new Date(-8.64e15)) == '"-271821-04-20T00:00:00.000Z"' &&
                  stringify(new Date(8.64e15)) == '"+275760-09-13T00:00:00.000Z"' &&
                  stringify(new Date(-621987552e5)) == '"-000001-01-01T00:00:00.000Z"' &&
                  stringify(new Date(-1)) == '"1969-12-31T23:59:59.999Z"';
              });
            }
          } else {
            var value, serialized = '{"a":[1,true,false,null,"\\u0000\\b\\n\\f\\r\\t"]}';
            if (name == "json-stringify") {
              var stringify = exports.stringify,
                stringifySupported = typeof stringify == "function";
              if (stringifySupported) {
                (value = function() {
                  return 1;
                }).toJSON = value;
                attempt(function() {
                  stringifySupported =
                    stringify(0) === "0" &&
                    stringify(new Number()) === "0" &&
                    stringify(new String()) == '""' &&
                    stringify(getClass) === undefined &&
                    stringify(undefined) === undefined &&
                    stringify() === undefined &&
                    stringify(value) === "1" &&
                    stringify([value]) == "[1]" &&
                    stringify([undefined]) == "[null]" &&
                    stringify(null) == "null" &&
                    stringify([undefined, getClass, null]) == "[null,null,null]" &&
                    stringify({
                      "a": [value, true, false, null, "\x00\b\n\f\r\t"]
                    }) == serialized &&
                    stringify(null, value) === "1" &&
                    stringify([1, 2], null, 1) == "[\n 1,\n 2\n]";
                }, function() {
                  stringifySupported = false;
                });
              }
              isSupported = stringifySupported;
            }
            if (name == "json-parse") {
              var parse = exports.parse,
                parseSupported;
              if (typeof parse == "function") {
                attempt(function() {
                  if (parse("0") === 0 && !parse(false)) {
                    value = parse(serialized);
                    parseSupported = value["a"].length == 5 && value["a"][0] === 1;
                    if (parseSupported) {
                      attempt(function() {
                        parseSupported = !parse('"\t"');
                      });
                      if (parseSupported) {
                        attempt(function() {
                          parseSupported = parse("01") !== 1;
                        });
                      }
                      if (parseSupported) {
                        attempt(function() {
                          parseSupported = parse("1.") !== 1;
                        });
                      }
                    }
                  }
                }, function() {
                  parseSupported = false;
                });
              }
              isSupported = parseSupported;
            }
          }
          return has[name] = !!isSupported;
        }
        has["bug-string-char-index"] = has["date-serialization"] = has["json"] = has["json-stringify"] = has["json-parse"] = null;

        if (!has("json")) {
          var functionClass = "[object Function]",
            dateClass = "[object Date]",
            numberClass = "[object Number]",
            stringClass = "[object String]",
            arrayClass = "[object Array]",
            booleanClass = "[object Boolean]";

          var charIndexBuggy = has("bug-string-char-index");

          var forOwn = function(object, callback) {
            var size = 0,
              Properties, dontEnums, property;

            (Properties = function() {
              this.valueOf = 0;
            }).prototype.valueOf = 0;

            dontEnums = new Properties();
            for (property in dontEnums) {
              if (isProperty.call(dontEnums, property)) {
                size++;
              }
            }
            Properties = dontEnums = null;

            if (!size) {
              dontEnums = ["valueOf", "toString", "toLocaleString", "propertyIsEnumerable", "isPrototypeOf", "hasOwnProperty", "constructor"];
              forOwn = function(object, callback) {
                var isFunction = getClass.call(object) == functionClass,
                  property, length;
                var hasProperty = !isFunction && typeof object.constructor != "function" && objectTypes[typeof object.hasOwnProperty] && object.hasOwnProperty || isProperty;
                for (property in object) {
                  if (!(isFunction && property == "prototype") && hasProperty.call(object, property)) {
                    callback(property);
                  }
                }
                for (length = dontEnums.length; property = dontEnums[--length];) {
                  if (hasProperty.call(object, property)) {
                    callback(property);
                  }
                }
              };
            } else {
              forOwn = function(object, callback) {
                var isFunction = getClass.call(object) == functionClass,
                  property, isConstructor;
                for (property in object) {
                  if (!(isFunction && property == "prototype") && isProperty.call(object, property) && !(isConstructor = property === "constructor")) {
                    callback(property);
                  }
                }
                if (isConstructor || isProperty.call(object, (property = "constructor"))) {
                  callback(property);
                }
              };
            }
            return forOwn(object, callback);
          };

          if (!has("json-stringify") && !has("date-serialization")) {
            var Escapes = {
              92: "\\\\",
              34: '\\"',
              8: "\\b",
              12: "\\f",
              10: "\\n",
              13: "\\r",
              9: "\\t"
            };

            var leadingZeroes = "000000";
            var toPaddedString = function(width, value) {
              return (leadingZeroes + (value || 0)).slice(-width);
            };

            var serializeDate = function(value) {
              var getData, year, month, date, time, hours, minutes, seconds, milliseconds;
              if (!isExtended) {
                var floor = Math.floor;
                var Months = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334];
                var getDay = function(year, month) {
                  return Months[month] + 365 * (year - 1970) + floor((year - 1969 + (month = +(month > 1))) / 4) - floor((year - 1901 + month) / 100) + floor((year - 1601 + month) / 400);
                };
                getData = function(value) {
                  date = floor(value / 864e5);
                  for (year = floor(date / 365.2425) + 1970 - 1; getDay(year + 1, 0) <= date; year++);
                  for (month = floor((date - getDay(year, 0)) / 30.42); getDay(year, month + 1) <= date; month++);
                  date = 1 + date - getDay(year, month);
                  time = (value % 864e5 + 864e5) % 864e5;
                  hours = floor(time / 36e5) % 24;
                  minutes = floor(time / 6e4) % 60;
                  seconds = floor(time / 1e3) % 60;
                  milliseconds = time % 1e3;
                };
              } else {
                getData = function(value) {
                  year = value.getUTCFullYear();
                  month = value.getUTCMonth();
                  date = value.getUTCDate();
                  hours = value.getUTCHours();
                  minutes = value.getUTCMinutes();
                  seconds = value.getUTCSeconds();
                  milliseconds = value.getUTCMilliseconds();
                };
              }
              serializeDate = function(value) {
                if (value > -1 / 0 && value < 1 / 0) {
                  getData(value);
                  value = (year <= 0 || year >= 1e4 ? (year < 0 ? "-" : "+") + toPaddedString(6, year < 0 ? -year : year) : toPaddedString(4, year)) +
                    "-" + toPaddedString(2, month + 1) + "-" + toPaddedString(2, date) +
                    "T" + toPaddedString(2, hours) + ":" + toPaddedString(2, minutes) + ":" + toPaddedString(2, seconds) +
                    "." + toPaddedString(3, milliseconds) + "Z";
                  year = month = date = hours = minutes = seconds = milliseconds = null;
                } else {
                  value = null;
                }
                return value;
              };
              return serializeDate(value);
            };

            if (has("json-stringify") && !has("date-serialization")) {
              function dateToJSON(key) {
                return serializeDate(this);
              }

              var nativeStringify = exports.stringify;
              exports.stringify = function(source, filter, width) {
                var nativeToJSON = Date.prototype.toJSON;
                Date.prototype.toJSON = dateToJSON;
                var result = nativeStringify(source, filter, width);
                Date.prototype.toJSON = nativeToJSON;
                return result;
              }
            } else {
              var unicodePrefix = "\\u00";
              var escapeChar = function(character) {
                var charCode = character.charCodeAt(0),
                  escaped = Escapes[charCode];
                if (escaped) {
                  return escaped;
                }
                return unicodePrefix + toPaddedString(2, charCode.toString(16));
              };
              var reEscape = /[\x00-\x1f\x22\x5c]/g;
              var quote = function(value) {
                reEscape.lastIndex = 0;
                return '"' +
                  (
                    reEscape.test(value) ?
                    value.replace(reEscape, escapeChar) :
                    value
                  ) +
                  '"';
              };

              var serialize = function(property, object, callback, properties, whitespace, indentation, stack) {
                var value, type, className, results, element, index, length, prefix, result;
                attempt(function() {
                  value = object[property];
                });
                if (typeof value == "object" && value) {
                  if (value.getUTCFullYear && getClass.call(value) == dateClass && value.toJSON === Date.prototype.toJSON) {
                    value = serializeDate(value);
                  } else if (typeof value.toJSON == "function") {
                    value = value.toJSON(property);
                  }
                }
                if (callback) {
                  value = callback.call(object, property, value);
                }
                if (value == undefined) {
                  return value === undefined ? value : "null";
                }
                type = typeof value;
                if (type == "object") {
                  className = getClass.call(value);
                }
                switch (className || type) {
                  case "boolean":
                  case booleanClass:
                    return "" + value;
                  case "number":
                  case numberClass:
                    return value > -1 / 0 && value < 1 / 0 ? "" + value : "null";
                  case "string":
                  case stringClass:
                    return quote("" + value);
                }
                if (typeof value == "object") {
                  for (length = stack.length; length--;) {
                    if (stack[length] === value) {
                      throw TypeError();
                    }
                  }
                  stack.push(value);
                  results = [];
                  prefix = indentation;
                  indentation += whitespace;
                  if (className == arrayClass) {
                    for (index = 0, length = value.length; index < length; index++) {
                      element = serialize(index, value, callback, properties, whitespace, indentation, stack);
                      results.push(element === undefined ? "null" : element);
                    }
                    result = results.length ? (whitespace ? "[\n" + indentation + results.join(",\n" + indentation) + "\n" + prefix + "]" : ("[" + results.join(",") + "]")) : "[]";
                  } else {
                    forOwn(properties || value, function(property) {
                      var element = serialize(property, value, callback, properties, whitespace, indentation, stack);
                      if (element !== undefined) {
                        results.push(quote(property) + ":" + (whitespace ? " " : "") + element);
                      }
                    });
                    result = results.length ? (whitespace ? "{\n" + indentation + results.join(",\n" + indentation) + "\n" + prefix + "}" : ("{" + results.join(",") + "}")) : "{}";
                  }
                  stack.pop();
                  return result;
                }
              };

              exports.stringify = function(source, filter, width) {
                var whitespace, callback, properties, className;
                if (objectTypes[typeof filter] && filter) {
                  className = getClass.call(filter);
                  if (className == functionClass) {
                    callback = filter;
                  } else if (className == arrayClass) {
                    properties = {};
                    for (var index = 0, length = filter.length, value; index < length;) {
                      value = filter[index++];
                      className = getClass.call(value);
                      if (className == "[object String]" || className == "[object Number]") {
                        properties[value] = 1;
                      }
                    }
                  }
                }
                if (width) {
                  className = getClass.call(width);
                  if (className == numberClass) {
                    if ((width -= width % 1) > 0) {
                      if (width > 10) {
                        width = 10;
                      }
                      for (whitespace = ""; whitespace.length < width;) {
                        whitespace += " ";
                      }
                    }
                  } else if (className == stringClass) {
                    whitespace = width.length <= 10 ? width : width.slice(0, 10);
                  }
                }
                return serialize("", (value = {}, value[""] = source, value), callback, properties, whitespace, "", []);
              };
            }
          }

          if (!has("json-parse")) {
            var fromCharCode = String.fromCharCode;

            var Unescapes = {
              92: "\\",
              34: '"',
              47: "/",
              98: "\b",
              116: "\t",
              110: "\n",
              102: "\f",
              114: "\r"
            };

            var Index, Source;

            var abort = function() {
              Index = Source = null;
              throw SyntaxError();
            };

            var lex = function() {
              var source = Source,
                length = source.length,
                value, begin, position, isSigned, charCode;
              while (Index < length) {
                charCode = source.charCodeAt(Index);
                switch (charCode) {
                  case 9:
                  case 10:
                  case 13:
                  case 32:
                    Index++;
                    break;
                  case 123:
                  case 125:
                  case 91:
                  case 93:
                  case 58:
                  case 44:
                    value = charIndexBuggy ? source.charAt(Index) : source[Index];
                    Index++;
                    return value;
                  case 34:
                    for (value = "@", Index++; Index < length;) {
                      charCode = source.charCodeAt(Index);
                      if (charCode < 32) {
                        abort();
                      } else if (charCode == 92) {
                        charCode = source.charCodeAt(++Index);
                        switch (charCode) {
                          case 92:
                          case 34:
                          case 47:
                          case 98:
                          case 116:
                          case 110:
                          case 102:
                          case 114:
                            value += Unescapes[charCode];
                            Index++;
                            break;
                          case 117:
                            begin = ++Index;
                            for (position = Index + 4; Index < position; Index++) {
                              charCode = source.charCodeAt(Index);
                              if (!(charCode >= 48 && charCode <= 57 || charCode >= 97 && charCode <= 102 || charCode >= 65 && charCode <= 70)) {
                                abort();
                              }
                            }
                            value += fromCharCode("0x" + source.slice(begin, Index));
                            break;
                          default:
                            abort();
                        }
                      } else {
                        if (charCode == 34) {
                          break;
                        }
                        charCode = source.charCodeAt(Index);
                        begin = Index;
                        while (charCode >= 32 && charCode != 92 && charCode != 34) {
                          charCode = source.charCodeAt(++Index);
                        }
                        value += source.slice(begin, Index);
                      }
                    }
                    if (source.charCodeAt(Index) == 34) {
                      Index++;
                      return value;
                    }
                    abort();
                  default:
                    begin = Index;
                    if (charCode == 45) {
                      isSigned = true;
                      charCode = source.charCodeAt(++Index);
                    }
                    if (charCode >= 48 && charCode <= 57) {
                      if (charCode == 48 && ((charCode = source.charCodeAt(Index + 1)), charCode >= 48 && charCode <= 57)) {
                        abort();
                      }
                      isSigned = false;
                      for (; Index < length && ((charCode = source.charCodeAt(Index)), charCode >= 48 && charCode <= 57); Index++);
                      if (source.charCodeAt(Index) == 46) {
                        position = ++Index;
                        for (; position < length; position++) {
                          charCode = source.charCodeAt(position);
                          if (charCode < 48 || charCode > 57) {
                            break;
                          }
                        }
                        if (position == Index) {
                          abort();
                        }
                        Index = position;
                      }
                      charCode = source.charCodeAt(Index);
                      if (charCode == 101 || charCode == 69) {
                        charCode = source.charCodeAt(++Index);
                        if (charCode == 43 || charCode == 45) {
                          Index++;
                        }
                        for (position = Index; position < length; position++) {
                          charCode = source.charCodeAt(position);
                          if (charCode < 48 || charCode > 57) {
                            break;
                          }
                        }
                        if (position == Index) {
                          abort();
                        }
                        Index = position;
                      }
                      return +source.slice(begin, Index);
                    }
                    if (isSigned) {
                      abort();
                    }
                    var temp = source.slice(Index, Index + 4);
                    if (temp == "true") {
                      Index += 4;
                      return true;
                    } else if (temp == "fals" && source.charCodeAt(Index + 4) == 101) {
                      Index += 5;
                      return false;
                    } else if (temp == "null") {
                      Index += 4;
                      return null;
                    }
                    abort();
                }
              }
              return "$";
            };

            var get = function(value) {
              var results, hasMembers;
              if (value == "$") {
                abort();
              }
              if (typeof value == "string") {
                if ((charIndexBuggy ? value.charAt(0) : value[0]) == "@") {
                  return value.slice(1);
                }
                if (value == "[") {
                  results = [];
                  for (;;) {
                    value = lex();
                    if (value == "]") {
                      break;
                    }
                    if (hasMembers) {
                      if (value == ",") {
                        value = lex();
                        if (value == "]") {
                          abort();
                        }
                      } else {
                        abort();
                      }
                    } else {
                      hasMembers = true;
                    }
                    if (value == ",") {
                      abort();
                    }
                    results.push(get(value));
                  }
                  return results;
                } else if (value == "{") {
                  results = {};
                  for (;;) {
                    value = lex();
                    if (value == "}") {
                      break;
                    }
                    if (hasMembers) {
                      if (value == ",") {
                        value = lex();
                        if (value == "}") {
                          abort();
                        }
                      } else {
                        abort();
                      }
                    } else {
                      hasMembers = true;
                    }
                    if (value == "," || typeof value != "string" || (charIndexBuggy ? value.charAt(0) : value[0]) != "@" || lex() != ":") {
                      abort();
                    }
                    results[value.slice(1)] = get(lex());
                  }
                  return results;
                }
                abort();
              }
              return value;
            };

            var update = function(source, property, callback) {
              var element = walk(source, property, callback);
              if (element === undefined) {
                delete source[property];
              } else {
                source[property] = element;
              }
            };

            var walk = function(source, property, callback) {
              var value = source[property],
                length;
              if (typeof value == "object" && value) {
                if (getClass.call(value) == arrayClass) {
                  for (length = value.length; length--;) {
                    update(getClass, forOwn, value, length, callback);
                  }
                } else {
                  forOwn(value, function(property) {
                    update(value, property, callback);
                  });
                }
              }
              return callback.call(source, property, value);
            };

            exports.parse = function(source, callback) {
              var result, value;
              Index = 0;
              Source = "" + source;
              result = get(lex());
              if (lex() != "$") {
                abort();
              }
              Index = Source = null;
              return callback && getClass.call(callback) == functionClass ? walk((value = {}, value[""] = result, value), "", callback) : result;
            };
          }
        }

        exports.runInContext = runInContext;
        return exports;
      }


      var nativeJSON = root.JSON,
        previousJSON = root.JSON3,
        isRestored = false;

      var JSON3 = runInContext(root, (root.JSON3 = {
        "noConflict": function() {
          if (!isRestored) {
            isRestored = true;
            root.JSON = nativeJSON;
            root.JSON3 = previousJSON;
            nativeJSON = previousJSON = null;
          }
          return JSON3;
        }
      }));

      root.JSON = {
        "parse": JSON3.parse,
        "stringify": JSON3.stringify
      };

    }).call(this);;


    if (!String.prototype.replaceAll) {
      String.prototype.replaceAll = function(str, newStr) {
        if (Object.prototype.toString.call(str).toLowerCase() === '[object regexp]') {
          return this.replace(str, newStr);
        }
        return this.replace(new RegExp(str, 'g'), newStr);
      };
    }

    (function() {
      var ArrayProto = Array.prototype;
      var FuncProto = Function.prototype;
      var ObjProto = Object.prototype;
      var slice = ArrayProto.slice;
      var toString = ObjProto.toString;
      var hasOwnProperty = ObjProto.hasOwnProperty;
      var nativeBind = FuncProto.bind;
      var nativeForEach = ArrayProto.forEach;
      var nativeIndexOf = ArrayProto.indexOf;
      var nativeIsArray = Array.isArray;
      var breaker = {};

      var each = (_.each = function(obj, iterator, context) {
        if (obj == null) {
          return false;
        }
        if (nativeForEach && obj.forEach === nativeForEach) {
          obj.forEach(iterator, context);
        } else if (_.isArray(obj) && obj.length === +obj.length) {
          for (var i = 0, l = obj.length; i < l; i++) {
            if (i in obj && iterator.call(context, obj[i], i, obj) === breaker) {
              return false;
            }
          }
        } else {
          for (var key in obj) {
            if (hasOwnProperty.call(obj, key)) {
              if (iterator.call(context, obj[key], key, obj) === breaker) {
                return false;
              }
            }
          }
        }
      });

      _.map = function(obj, iterator) {
        var results = [];
        if (obj == null) {
          return results;
        }
        if (Array.prototype.map && obj.map === Array.prototype.map) {
          return obj.map(iterator);
        }
        each(obj, function(value, index, list) {
          results.push(iterator(value, index, list));
        });
        return results;
      };

      _.extend = function(obj) {
        each(slice.call(arguments, 1), function(source) {
          for (var prop in source) {
            if (hasOwnProperty.call(source, prop) && source[prop] !== void 0) {
              obj[prop] = source[prop];
            }
          }
        });
        return obj;
      };

      _.extend2Lev = function(obj) {
        each(slice.call(arguments, 1), function(source) {
          for (var prop in source) {
            if (source[prop] !== void 0) {
              if (_.isObject(source[prop]) && _.isObject(obj[prop])) {
                _.extend(obj[prop], source[prop]);
              } else {
                obj[prop] = source[prop];
              }
            }
          }
        });
        return obj;
      };
      _.coverExtend = function(obj) {
        each(slice.call(arguments, 1), function(source) {
          for (var prop in source) {
            if (source[prop] !== void 0 && obj[prop] === void 0) {
              obj[prop] = source[prop];
            }
          }
        });
        return obj;
      };

      _.isArray =
        nativeIsArray ||
        function(obj) {
          return toString.call(obj) === '[object Array]';
        };

      _.isFunction = function(f) {
        if (!f) {
          return false;
        }
        try {
          return /^\s*\bfunction\b/.test(f);
        } catch (x) {
          return false;
        }
      };

      _.isArguments = function(obj) {
        return !!(obj && hasOwnProperty.call(obj, 'callee'));
      };

      _.toArray = function(iterable) {
        if (!iterable) {
          return [];
        }
        if (iterable.toArray) {
          return iterable.toArray();
        }
        if (_.isArray(iterable)) {
          return slice.call(iterable);
        }
        if (_.isArguments(iterable)) {
          return slice.call(iterable);
        }
        return _.values(iterable);
      };

      _.values = function(obj) {
        var results = [];
        if (obj == null) {
          return results;
        }
        each(obj, function(value) {
          results[results.length] = value;
        });
        return results;
      };

      _.indexOf = function(arr, target) {
        var indexof = arr.indexOf;
        if (indexof) {
          return indexof.call(arr, target);
        } else {
          for (var i = 0; i < arr.length; i++) {
            if (target === arr[i]) {
              return i;
            }
          }
          return -1;
        }
      };

      _.hasAttributes = function(ele, attrs) {
        if (typeof attrs === 'string') {
          return _.hasAttribute(ele, attrs);
        } else if (_.isArray(attrs)) {
          var result = false;
          for (var i = 0; i < attrs.length; i++) {
            var testResult = _.hasAttribute(ele, attrs[i]);
            if (testResult) {
              result = true;
              break;
            }
          }
          return result;
        }
      };

      _.hasAttribute = function(ele, attr) {
        if (ele.hasAttribute) {
          return ele.hasAttribute(attr);
        } else {
          return !!(ele.attributes[attr] && ele.attributes[attr].specified);
        }
      };

      _.filter = function(arr, fn, self) {
        var hasOwn = Object.prototype.hasOwnProperty;
        if (arr.filter) {
          return arr.filter(fn);
        }
        var ret = [];
        for (var i = 0; i < arr.length; i++) {
          if (!hasOwn.call(arr, i)) {
            continue;
          }
          var val = arr[i];
          if (fn.call(self, val, i, arr)) {
            ret.push(val);
          }
        }
        return ret;
      };

      _.inherit = function(subclass, superclass) {
        subclass.prototype = new superclass();
        subclass.prototype.constructor = subclass;
        subclass.superclass = superclass.prototype;
        return subclass;
      };

      _.trim = function(str) {
        return str.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
      };

      _.isObject = function(obj) {
        if (obj == null) {
          return false;
        } else {
          return toString.call(obj) == '[object Object]';
        }
      };

      _.isEmptyObject = function(obj) {
        if (_.isObject(obj)) {
          for (var key in obj) {
            if (hasOwnProperty.call(obj, key)) {
              return false;
            }
          }
          return true;
        }
        return false;
      };

      _.isUndefined = function(obj) {
        return obj === void 0;
      };

      _.isString = function(obj) {
        return toString.call(obj) == '[object String]';
      };

      _.isDate = function(obj) {
        return toString.call(obj) == '[object Date]';
      };

      _.isBoolean = function(obj) {
        return toString.call(obj) == '[object Boolean]';
      };

      _.isNumber = function(obj) {
        return toString.call(obj) == '[object Number]' && /[\d\.]+/.test(String(obj));
      };

      _.isElement = function(obj) {
        return !!(obj && obj.nodeType === 1);
      };

      _.isJSONString = function(str) {
        try {
          JSON.parse(str);
        } catch (e) {
          return false;
        }
        return true;
      };
      _.safeJSONParse = function(str) {
        var val = null;
        try {
          val = JSON.parse(str);
        } catch (e) {
          return false;
        }
        return val;
      };
      _.decodeURIComponent = function(val) {
        var result = val;
        try {
          result = decodeURIComponent(val);
        } catch (e) {
          result = val;
        }
        return result;
      };

      _.decodeURI = function(val) {
        var result = val;
        try {
          result = decodeURI(val);
        } catch (e) {
          result = val;
        }
        return result;
      };

      _.isDecodeURI = function(para, val) {
        if (para) {
          return _.decodeURI(val);
        } else {
          return val;
        }
      };

      _.encodeDates = function(obj) {
        _.each(obj, function(v, k) {
          if (_.isDate(v)) {
            obj[k] = _.formatDate(v);
          } else if (_.isObject(v)) {
            obj[k] = _.encodeDates(v);
          }
        });
        return obj;
      };

      _.mediaQueriesSupported = function() {
        return typeof window.matchMedia != 'undefined' || typeof window.msMatchMedia != 'undefined';
      };

      _.getScreenOrientation = function() {
        var screenOrientationAPI = screen.msOrientation || screen.mozOrientation || (screen.orientation || {}).type;
        var screenOrientation = '未取到值';
        if (screenOrientationAPI) {
          screenOrientation = screenOrientationAPI.indexOf('landscape') > -1 ? 'landscape' : 'portrait';
        } else if (_.mediaQueriesSupported()) {
          var matchMediaFunc = window.matchMedia || window.msMatchMedia;
          if (matchMediaFunc('(orientation: landscape)').matches) {
            screenOrientation = 'landscape';
          } else if (matchMediaFunc('(orientation: portrait)').matches) {
            screenOrientation = 'portrait';
          }
        }
        return screenOrientation;
      };

      _.now =
        Date.now ||
        function() {
          return new Date().getTime();
        };

      _.throttle = function(func, wait, options) {
        var context, args, result;
        var timeout = null;
        var previous = 0;
        if (!options) options = {};
        var later = function() {
          previous = options.leading === false ? 0 : _.now();
          timeout = null;
          result = func.apply(context, args);
          if (!timeout) context = args = null;
        };
        return function() {
          var now = _.now();
          if (!previous && options.leading === false) previous = now;
          var remaining = wait - (now - previous);
          context = this;
          args = arguments;
          if (remaining <= 0 || remaining > wait) {
            if (timeout) {
              clearTimeout(timeout);
              timeout = null;
            }
            previous = now;
            result = func.apply(context, args);
            if (!timeout) context = args = null;
          } else if (!timeout && options.trailing !== false) {
            timeout = setTimeout(later, remaining);
          }
          return result;
        };
      };

      _.hashCode = function(str) {
        if (typeof str !== 'string') {
          return 0;
        }
        var hash = 0;
        var char = null;
        if (str.length == 0) {
          return hash;
        }
        for (var i = 0; i < str.length; i++) {
          char = str.charCodeAt(i);
          hash = (hash << 5) - hash + char;
          hash = hash & hash;
        }
        return hash;
      };

      _.formatDate = function(d) {
        function pad(n) {
          return n < 10 ? '0' + n : n;
        }

        return d.getFullYear() + '-' + pad(d.getMonth() + 1) + '-' + pad(d.getDate()) + ' ' + pad(d.getHours()) + ':' + pad(d.getMinutes()) + ':' + pad(d.getSeconds()) + '.' + pad(d.getMilliseconds());
      };

      _.getRandomBasic = (function() {
        var today = new Date();
        var seed = today.getTime();

        function rnd() {
          seed = (seed * 9301 + 49297) % 233280;
          return seed / (233280.0);
        };
        return function rand(number) {
          return Math.ceil(rnd(seed) * number);
        };
      })();
      _.getRandom = function() {
        if (typeof Uint32Array === 'function') {
          var cry = '';
          if (typeof crypto !== 'undefined') {
            cry = crypto;
          } else if (typeof msCrypto !== 'undefined') {
            cry = msCrypto;
          }
          if (_.isObject(cry) && cry.getRandomValues) {
            var typedArray = new Uint32Array(1);
            var randomNumber = cry.getRandomValues(typedArray)[0];
            var integerLimit = Math.pow(2, 32);
            return randomNumber / integerLimit;
          }
        }
        return _.getRandomBasic(10000000000000000000) / 10000000000000000000;
      };

      _.searchObjDate = function(o) {
        if (_.isObject(o)) {
          _.each(o, function(a, b) {
            if (_.isObject(a)) {
              _.searchObjDate(o[b]);
            } else {
              if (_.isDate(a)) {
                o[b] = _.formatDate(a);
              }
            }
          });
        }
      };

      _.searchZZAppStyle = function(data) {
        if (typeof data.properties.$project !== 'undefined') {
          data.project = data.properties.$project;
          delete data.properties.$project;
        }
        if (typeof data.properties.$token !== 'undefined') {
          data.token = data.properties.$token;
          delete data.properties.$token;
        }
      };

      _.formatJsonString = function(obj) {
        try {
          return JSON.stringify(obj, null, '  ');
        } catch (e) {
          return JSON.stringify(obj);
        }
      };

      _.formatString = function(str, maxLen) {
        if (_.isNumber(maxLen) && str.length > maxLen) {
          webinsight.log('字符串长度超过限制，已经做截取--' + str);
          return str.slice(0, maxLen);
        } else {
          return str;
        }
      };

      _.searchObjString = function(o) {
        if (_.isObject(o)) {
          _.each(o, function(a, b) {
            if (_.isObject(a)) {
              _.searchObjString(o[b]);
            } else {
              if (_.isString(a)) {
                o[b] = _.formatString(a, b === '$element_selector' ? 1024 : webinsight.para.max_string_length);
              }
            }
          });
        }
      };

      _.parseSuperProperties = function(data) {
        var obj = data.properties;
        var copyData = JSON.parse(JSON.stringify(data));
        if (_.isObject(obj)) {
          _.each(obj, function(value, key) {
            if (_.isFunction(value)) {
              try {
                obj[key] = value(copyData);
                if (_.isFunction(obj[key])) {
                  webinsight.log('您的属性- ' + key + ' 格式不满足要求，我们已经将其删除');
                  delete obj[key];
                }
              } catch (e) {
                delete obj[key];
                webinsight.log('您的属性- ' + key + ' 抛出了异常，我们已经将其删除');
              }
            }
          });
          _.strip_sa_properties(obj);
        }
      };

      _.filterReservedProperties = function(obj) {
        var reservedFields = ['distinct_id', 'user_id', 'id', 'date', 'datetime', 'event', 'events', 'first_id', 'original_id', 'device_id', 'properties', 'second_id', 'time', 'users'];
        if (!_.isObject(obj)) {
          return;
        }
        _.each(reservedFields, function(key, index) {
          if (!(key in obj)) {
            return;
          }
          if (index < 3) {
            delete obj[key];
            webinsight.log('您的属性- ' + key + '是保留字段，我们已经将其删除');
          } else {
            webinsight.log('您的属性- ' + key + '是保留字段，请避免其作为属性名');
          }
        });
      };

      _.searchConfigData = function(data) {
        if (typeof data === 'object' && data.$option) {
          var data_config = data.$option;
          delete data.$option;
          return data_config;
        } else {
          return {};
        }
      };

      _.unique = function(ar) {
        var temp,
          n = [],
          o = {};
        for (var i = 0; i < ar.length; i++) {
          temp = ar[i];
          if (!(temp in o)) {
            o[temp] = true;
            n.push(temp);
          }
        }
        return n;
      };

      _.strip_sa_properties = function(p) {
        if (!_.isObject(p)) {
          return p;
        }
        _.each(p, function(v, k) {
          if (_.isArray(v)) {
            var temp = [];
            _.each(v, function(arrv) {
              if (_.isString(arrv)) {
                temp.push(arrv);
              } else {
                webinsight.log('您的数据-', k, v, '的数组里的值必须是字符串,已经将其删除');
              }
            });
            p[k] = temp;
          }
          if (!(_.isString(v) || _.isNumber(v) || _.isDate(v) || _.isBoolean(v) || _.isArray(v) || _.isFunction(v) || k === '$option')) {
            webinsight.log('您的数据-', k, v, '-格式不满足要求，我们已经将其删除');
            delete p[k];
          }
        });
        return p;
      };

      _.strip_empty_properties = function(p) {
        var ret = {};
        _.each(p, function(v, k) {
          if (v != null) {
            ret[k] = v;
          }
        });
        return ret;
      };

      _.UUID = (function() {
        var T = function() {
          var d = 1 * new Date(),
            i = 0;
          while (d == 1 * new Date()) {
            i++;
          }
          return d.toString(16) + i.toString(16);
        };
        var R = function() {
          return _.getRandom().toString(16).replace('.', '');
        };
        var UA = function(n) {
          var ua = navigator.userAgent,
            i,
            ch,
            buffer = [],
            ret = 0;

          function xor(result, byte_array) {
            var j,
              tmp = 0;
            for (j = 0; j < byte_array.length; j++) {
              tmp |= buffer[j] << (j * 8);
            }
            return result ^ tmp;
          }

          for (i = 0; i < ua.length; i++) {
            ch = ua.charCodeAt(i);
            buffer.unshift(ch & 0xff);
            if (buffer.length >= 4) {
              ret = xor(ret, buffer);
              buffer = [];
            }
          }

          if (buffer.length > 0) {
            ret = xor(ret, buffer);
          }

          return ret.toString(16);
        };

        return function() {
          var se = String(screen.height * screen.width);
          if (se && /\d{5,}/.test(se)) {
            se = se.toString(16);
          } else {
            se = String(_.getRandom() * 31242)
              .replace('.', '')
              .slice(0, 8);
          }
          var val = T() + '-' + R() + '-' + UA() + '-' + se + '-' + T();
          if (val) {
            return val;
          } else {
            return (String(_.getRandom()) + String(_.getRandom()) + String(_.getRandom())).slice(2, 15);
          }
        };
      })();

      _.getQueryParam = function(url, param) {
        param = param.replace(/[\[]/, '\\[').replace(/[\]]/, '\\]');
        url = _.decodeURIComponent(url);
        var regexS = '[\\?&]' + param + '=([^&#]*)',
          regex = new RegExp(regexS),
          results = regex.exec(url);
        if (results === null || (results && typeof results[1] !== 'string' && results[1].length)) {
          return '';
        } else {
          return _.decodeURIComponent(results[1]);
        }
      };

      _.urlParse = function(para) {
        var URLParser = function(a) {
          this._fields = {
            Username: 4,
            Password: 5,
            Port: 7,
            Protocol: 2,
            Host: 6,
            Path: 8,
            URL: 0,
            QueryString: 9,
            Fragment: 10
          };
          this._values = {};
          this._regex = null;
          this._regex = /^((\w+):\/\/)?((\w+):?(\w+)?@)?([^\/\?:]+):?(\d+)?(\/?[^\?#]+)?\??([^#]+)?#?(\w*)/;

          if (typeof a != 'undefined') {
            this._parse(a);
          }
        };
        URLParser.prototype.setUrl = function(a) {
          this._parse(a);
        };
        URLParser.prototype._initValues = function() {
          for (var a in this._fields) {
            this._values[a] = '';
          }
        };
        URLParser.prototype.addQueryString = function(queryObj) {
          if (typeof queryObj !== 'object') {
            return false;
          }
          var query = this._values.QueryString || '';
          for (var i in queryObj) {
            if (new RegExp(i + '[^&]+').test(query)) {
              query = query.replace(new RegExp(i + '[^&]+'), i + '=' + queryObj[i]);
            } else {
              if (query.slice(-1) === '&') {
                query = query + i + '=' + queryObj[i];
              } else {
                if (query === '') {
                  query = i + '=' + queryObj[i];
                } else {
                  query = query + '&' + i + '=' + queryObj[i];
                }
              }
            }
          }
          this._values.QueryString = query;
        };
        URLParser.prototype.getUrl = function() {
          var url = '';
          url += this._values.Origin;
          url += this._values.Port ? ':' + this._values.Port : '';
          url += this._values.Path;
          url += this._values.QueryString ? '?' + this._values.QueryString : '';
          url += this._values.Fragment ? '#' + this._values.Fragment : '';
          return url;
        };

        URLParser.prototype.getUrl = function() {
          var url = '';
          url += this._values.Origin;
          url += this._values.Port ? ':' + this._values.Port : '';
          url += this._values.Path;
          url += this._values.QueryString ? '?' + this._values.QueryString : '';
          return url;
        };
        URLParser.prototype._parse = function(a) {
          this._initValues();
          var b = this._regex.exec(a);
          if (!b) {
            webinsight.log('DPURLParser::_parse -> Invalid URL');
          }
          for (var c in this._fields) {
            if (typeof b[this._fields[c]] != 'undefined') {
              this._values[c] = b[this._fields[c]];
            }
          }
          this._values['Hostname'] = this._values['Host'].replace(/:\d+$/, '');
          this._values['Origin'] = this._values['Protocol'] + '://' + this._values['Hostname'];
        };
        return new URLParser(para);
      };


      _.addEvent = function() {
        function fixEvent(event) {
          if (event) {
            event.preventDefault = fixEvent.preventDefault;
            event.stopPropagation = fixEvent.stopPropagation;
            event._getPath = fixEvent._getPath;
          }
          return event;
        }
        fixEvent._getPath = function() {
          var ev = this;
          var polyfill = function() {
            try {
              var element = ev.target;
              var pathArr = [element];
              if (element === null || element.parentElement === null) {
                return [];
              }
              while (element.parentElement !== null) {
                element = element.parentElement;
                pathArr.unshift(element);
              }
              return pathArr;
            } catch (err) {
              return [];
            }
          };
          return this.path || (this.composedPath && this.composedPath()) || polyfill();
        };
        fixEvent.preventDefault = function() {
          this.returnValue = false;
        };
        fixEvent.stopPropagation = function() {
          this.cancelBubble = true;
        };

        var register_event = function(element, type, handler) {
          var useCapture = _.isObject(webinsight.para.heatmap) && webinsight.para.heatmap.useCapture ? true : false;
          if (_.isObject(webinsight.para.heatmap) && typeof webinsight.para.heatmap.useCapture === 'undefined' && type === 'click') {
            useCapture = true;
          }
          if (element && element.addEventListener) {
            element.addEventListener(
              type,
              function(e) {
                e._getPath = fixEvent._getPath;
                handler.call(this, e);
              },
              useCapture
            );
          } else {
            var ontype = 'on' + type;
            var old_handler = element[ontype];
            element[ontype] = makeHandler(element, handler, old_handler);
          }
        };

        function makeHandler(element, new_handler, old_handlers) {
          var handler = function(event) {
            event = event || fixEvent(window.event);
            if (!event) {
              return undefined;
            }
            event.target = event.srcElement;

            var ret = true;
            var old_result, new_result;
            if (typeof old_handlers === 'function') {
              old_result = old_handlers(event);
            }
            new_result = new_handler.call(element, event);
            if (false === old_result || false === new_result) {
              ret = false;
            }
            return ret;
          };
          return handler;
        }

        register_event.apply(null, arguments);
      };

      _.addHashEvent = function(callback) {
        var hashEvent = 'pushState' in window.history ? 'popstate' : 'hashchange';
        _.addEvent(window, hashEvent, callback);
      };

      _.addSinglePageEvent = function(callback) {
        var current_url = location.href;
        var historyPushState = window.history.pushState;
        var historyReplaceState = window.history.replaceState;

        window.history.pushState = function() {
          historyPushState.apply(window.history, arguments);
          callback(current_url);
          current_url = location.href;
        };
        window.history.replaceState = function() {
          historyReplaceState.apply(window.history, arguments);
          callback(current_url);
          current_url = location.href;
        };

        var singlePageEvent = historyPushState ? 'popstate' : 'hashchange';
        _.addEvent(window, singlePageEvent, function() {
          callback(current_url);
          current_url = location.href;
        });
      };

      _.cookie = {
        get: function(name) {
          var nameEQ = name + '=';
          var ca = document.cookie.split(';');
          for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') {
              c = c.substring(1, c.length);
            }
            if (c.indexOf(nameEQ) == 0) {
              return _.decodeURIComponent(c.substring(nameEQ.length, c.length));
            }
          }
          return null;
        },
        set: function(name, value, days, cross_subdomain) {
          cross_subdomain = typeof cross_subdomain === 'undefined' ? webinsight.para.cross_subdomain : cross_subdomain;
          var cdomain = '',
            expires = '',
            secure = '';
          days = days == null ? 73000 : days;

          if (cross_subdomain) {
            var domain = _.getCurrentDomain(location.href);
            if (domain === 'url解析失败') {
              domain = '';
            }
            cdomain = domain ? '; domain=' + domain : '';
          }

          if (days !== 0) {
            var date = new Date();
            if (String(days).slice(-1) === 's') {
              date.setTime(date.getTime() + Number(String(days).slice(0, -1)) * 1000);
            } else {
              date.setTime(date.getTime() + days * 24 * 60 * 60 * 1000);
            }

            expires = '; expires=' + date.toGMTString();
          }
          if (webinsight.para.is_secure_cookie) {
            secure = '; secure';
          }

          function getValid(data) {
            if (data) {
              return data.replaceAll(/\r\n/g, '');
            } else {
              return false;
            }
          }
          var valid_name = '';
          var valid_value = '';
          var valid_domain = '';
          if (name) {
            valid_name = getValid(name);
          }
          if (value) {
            valid_value = getValid(value);
          }
          if (cdomain) {
            valid_domain = getValid(cdomain);
          }
          if (valid_name && valid_value) {
            document.cookie = valid_name + '=' + encodeURIComponent(valid_value) + expires + '; path=/' + valid_domain + secure;
          }

        },
        encrypt: function(v) {
          return 'data:enc;' + _.rot13obfs(v);
        },
        decrypt: function(v) {
          v = v.substring('data:enc;'.length);
          v = _.rot13defs(v);
          return v;
        },
        resolveValue: function(cross) {
          var flag = 'data:enc;';
          if (_.isString(cross) && cross.indexOf(flag) === 0) {
            cross = _.cookie.decrypt(cross);
          }
          return cross;
        },

        remove: function(name, cross_subdomain) {
          cross_subdomain = typeof cross_subdomain === 'undefined' ? webinsight.para.cross_subdomain : cross_subdomain;
          _.cookie.set(name, '', -1, cross_subdomain);
        },

        getCookieName: function(name_prefix, url) {
          var sub = '';
          url = url || location.href;
          if (webinsight.para.cross_subdomain === false) {
            try {
              sub = _.URL(url).hostname;
            } catch (e) {
              webinsight.log(e);
            }
            if (typeof sub === 'string' && sub !== '') {
              sub = 'sajssdk_2015_' + name_prefix + '_' + sub.replace(/\./g, '_');
            } else {
              sub = 'sajssdk_2015_root_' + name_prefix;
            }
          } else {
            sub = 'sajssdk_2015_cross_' + name_prefix;
          }
          return sub;
        },
        getNewUser: function() {
          return false;
        }
      };
      _.getElementContent = function(target, tagName) {
        var textContent = '';
        var element_content = '';
        if (target.textContent) {
          textContent = _.trim(target.textContent);
        } else if (target.innerText) {
          textContent = _.trim(target.innerText);
        }
        if (textContent) {
          textContent = textContent
            .replace(/[\r\n]/g, ' ')
            .replace(/[ ]+/g, ' ')
            .substring(0, 255);
        }
        element_content = textContent || '';

        if (tagName === 'input' || tagName === 'INPUT') {
          if (target.type === 'button' || target.type === 'submit') {
            element_content = target.value || '';
          } else if (webinsight.para.heatmap && typeof webinsight.para.heatmap.collect_input === 'function' && webinsight.para.heatmap.collect_input(target)) {
            element_content = target.value || '';
          }
        }
        return element_content;
      };
      _.getEleInfo = function(obj) {
        if (!obj.target) {
          return false;
        }

        var target = obj.target;
        var tagName = target.tagName.toLowerCase();

        var props = {};

        props.$elementType = tagName;
        props.$elementName = target.getAttribute('name');
        props.$elementId = target.getAttribute('id');
        props.$elementClassName = typeof target.className === 'string' ? target.className : null;
        props.$elementTargetURL = target.getAttribute('href');
        props.$elementContent = _.getElementContent(target, tagName);

        props = _.strip_empty_properties(props);

        (props.$url = _.isDecodeURI(webinsight.para.url_is_decode, location.href)), (props.$urlPath = location.pathname);
        props.$title = document.title;
        props.$viewportWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth || 0;
        props.$viewportHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight || 0;

        props.$urlHost = _.getHostname(props.$url,'urlHost取值异常')

        return props;
      };

      _.getHostname = function(url, defaultValue) {
        if (!defaultValue || typeof defaultValue !== 'string') {
          defaultValue = 'hostname解析异常';
        }
        var hostname = null;
        try {
          hostname = _.URL(url).hostname;
        } catch (e) {
          webinsight.log('getHostname传入的url参数不合法！');
        }
        return hostname || defaultValue;
      };

      _.getQueryParamsFromUrl = function(url) {
        var result = {};
        var arr = url.split('?');
        var queryString = arr[1] || '';
        if (queryString) {
          result = _.getURLSearchParams('?' + queryString);
        }
        return result;
      };

      _.getURLSearchParams = function(queryString) {
        queryString = queryString || '';
        var decodeParam = function(str) {
          return decodeURIComponent(str);
        };
        var args = {};
        var query = queryString.substring(1);
        var pairs = query.split('&');
        for (var i = 0; i < pairs.length; i++) {
          var pos = pairs[i].indexOf('=');
          if (pos === -1) continue;
          var name = pairs[i].substring(0, pos);
          var value = pairs[i].substring(pos + 1);
          name = decodeParam(name);
          value = decodeParam(value);
          args[name] = value;
        }
        return args;
      };

      _.URL = function(url) {
        var result = {};
        var basicProps = ['hash', 'host', 'hostname', 'href', 'origin', 'password', 'pathname', 'port', 'protocol', 'search', 'username'];
        var isURLAPIWorking = function() {
          return false
        };
        if (typeof window.URL === 'function' && isURLAPIWorking()) {
          result = new URL(url);
          if (!result.searchParams) {
            result.searchParams = (function() {
              var params = _.getURLSearchParams(result.search);
              return {
                get: function(searchParam) {
                  return params[searchParam];
                }
              };
            })();
          }
        } else {
          var _regex = /^https?:\/\/.+/;
          if (_regex.test(url) === false) {
            webinsight.log('Invalid URL');
          }
          var instance = _.urlParse(url);
          result.hash = '';
          result.host = instance._values.Host ? instance._values.Host + (instance._values.Port ? ':' + instance._values.Port : '') : '';
          result.href = instance._values.URL;
          result.password = instance._values.Password;
          result.pathname = instance._values.Path;
          result.port = instance._values.Port;
          result.search = instance._values.QueryString ? '?' + instance._values.QueryString : '';
          result.username = instance._values.Username;
          result.hostname = instance._values.Hostname;
          result.protocol = instance._values.Protocol ? instance._values.Protocol + ':' : '';
          result.origin = instance._values.Origin ? instance._values.Origin + (instance._values.Port ? ':' + instance._values.Port : '') : '';
          result.searchParams = (function() {
            var params = _.getURLSearchParams('?' + instance._values.QueryString);
            return {
              get: function(searchParam) {
                return params[searchParam];
              }
            };
          })();
        }
        return result;
      };

      _.getCurrentDomain = function(url) {
        var sdDomain = webinsight.para.current_domain;
        switch (typeof sdDomain) {
          case 'function':
            var resultDomain = sdDomain();
            if (resultDomain === '' || _.trim(resultDomain) === '') {
              return 'url解析失败';
            } else if (resultDomain.indexOf('.') !== -1) {
              return resultDomain;
            } else {
              return 'url解析失败';
            }
            case 'string':
              if (sdDomain === '' || _.trim(sdDomain) === '') {
                return 'url解析失败';
              } else if (sdDomain.indexOf('.') !== -1) {
                return sdDomain;
              } else {
                return 'url解析失败';
              }
              default:
                var cookieTopLevelDomain = _.getCookieTopLevelDomain();
                if (url === '') {
                  return 'url解析失败';
                } else if (cookieTopLevelDomain === '') {
                  return 'url解析失败';
                } else {
                  return cookieTopLevelDomain;
                }
        }
      };

      _.getCookieTopLevelDomain = function(hostname) {
        hostname = hostname || location.hostname;

        function validHostname(value) {
          if (value) {
            return value;
          } else {
            return false;
          }
        }
        var new_hostname = validHostname(hostname);
        if (!new_hostname) {
          return '';
        }
        var splitResult = new_hostname.split('.');
        if (_.isArray(splitResult) && splitResult.length >= 2 && !/^(\d+\.)+\d+$/.test(new_hostname)) {
          var domainStr = '.' + splitResult.splice(splitResult.length - 1, 1);
          while (splitResult.length > 0) {
            domainStr = '.' + splitResult.splice(splitResult.length - 1, 1) + domainStr;
            document.cookie = 'webinsightdata_domain_test=true; path=/; domain=' + domainStr;

            if (document.cookie.indexOf('webinsightdata_domain_test=true') !== -1) {
              var now = new Date();
              now.setTime(now.getTime() - 1000);

              document.cookie = 'webinsightdata_domain_test=true; expires=' + now.toGMTString() + '; path=/; domain=' + domainStr;

              return domainStr;
            }
          }
        }
        return '';
      };

      _.isReferralTraffic = function(refererstring) {
        refererstring = refererstring || document.referrer;
        if (refererstring === '') {
          return true;
        }

        return _.getCookieTopLevelDomain(_.getHostname(refererstring)) !== _.getCookieTopLevelDomain();
      };

      _.ry = function(dom) {
        return new _.ry.init(dom);
      };
      _.ry.init = function(dom) {
        this.ele = dom;
      };
      _.ry.init.prototype = {
        addClass: function(para) {
          var classes = ' ' + this.ele.className + ' ';
          if (classes.indexOf(' ' + para + ' ') === -1) {
            this.ele.className = this.ele.className + (this.ele.className === '' ? '' : ' ') + para;
          }
          return this;
        },
        removeClass: function(para) {
          var classes = ' ' + this.ele.className + ' ';
          if (classes.indexOf(' ' + para + ' ') !== -1) {
            this.ele.className = classes.replace(' ' + para + ' ', ' ').slice(1, -1);
          }
          return this;
        },
        hasClass: function(para) {
          var classes = ' ' + this.ele.className + ' ';
          if (classes.indexOf(' ' + para + ' ') !== -1) {
            return true;
          } else {
            return false;
          }
        },
        attr: function(key, value) {
          if (typeof key === 'string' && _.isUndefined(value)) {
            return this.ele.getAttribute(key);
          }
          if (typeof key === 'string') {
            value = String(value);
            this.ele.setAttribute(key, value);
          }
          return this;
        },
        offset: function() {
          var rect = this.ele.getBoundingClientRect();
          if (rect.width || rect.height) {
            var doc = this.ele.ownerDocument;
            var docElem = doc.documentElement;

            return {
              top: rect.top + window.pageYOffset - docElem.clientTop,
              left: rect.left + window.pageXOffset - docElem.clientLeft
            };
          } else {
            return {
              top: 0,
              left: 0
            };
          }
        },
        getSize: function() {
          if (!window.getComputedStyle) {
            return {
              width: this.ele.offsetWidth,
              height: this.ele.offsetHeight
            };
          }
          try {
            var bounds = this.ele.getBoundingClientRect();
            return {
              width: bounds.width,
              height: bounds.height
            };
          } catch (e) {
            return {
              width: 0,
              height: 0
            };
          }
        },
        getStyle: function(value) {
          if (this.ele.currentStyle) {
            return this.ele.currentStyle[value];
          } else {
            return this.ele.ownerDocument.defaultView.getComputedStyle(this.ele, null).getPropertyValue(value);
          }
        },
        wrap: function(elementTagName) {
          var ele = document.createElement(elementTagName);
          this.ele.parentNode.insertBefore(ele, this.ele);
          ele.appendChild(this.ele);
          return _.ry(ele);
        },
        getCssStyle: function(prop) {
          var result = this.ele.style.getPropertyValue(prop);
          if (result) {
            return result;
          }
          var rules = null;
          if (typeof window.getMatchedCSSRules === 'function') {
            rules = getMatchedCSSRules(this.ele);
          }
          if (!rules || !_.isArray(rules)) {
            return null;
          }
          for (var i = rules.length - 1; i >= 0; i--) {
            var r = rules[i];
            result = r.style.getPropertyValue(prop);
            if (result) {
              return result;
            }
          }
        },
        sibling: function(cur, dir) {
          while ((cur = cur[dir]) && cur.nodeType !== 1) {}
          return cur;
        },
        next: function() {
          return this.sibling(this.ele, 'nextSibling');
        },
        prev: function(elem) {
          return this.sibling(this.ele, 'previousSibling');
        },
        siblings: function(elem) {
          return this.siblings((this.ele.parentNode || {}).firstChild, this.ele);
        },
        children: function(elem) {
          return this.siblings(this.ele.firstChild);
        },
        parent: function() {
          var parent = this.ele.parentNode;
          parent = parent && parent.nodeType !== 11 ? parent : null;
          return _.ry(parent);
        },
        previousElementSibling: function() {
          var el = this.ele;
          if ('previousElementSibling' in document.documentElement) {
            return _.ry(el.previousElementSibling);
          } else {
            while ((el = el.previousSibling)) {
              if (el.nodeType === 1) {
                return _.ry(el);
              }
            }
            return _.ry(null);
          }
        },
        getSameTypeSiblings: function() {
          var element = this.ele;
          var parentNode = element.parentNode;
          var tagName = element.tagName.toLowerCase();
          var arr = [];
          for (var i = 0; i < parentNode.children.length; i++) {
            var child = parentNode.children[i];
            if (child.nodeType === 1 && child.tagName.toLowerCase() === tagName) {
              arr.push(parentNode.children[i]);
            }
          }
          return arr;
        }
      };

      _.strToUnicode = function(str) {
        if (typeof str !== 'string') {
          webinsight.log('转换unicode错误', str);
          return str;
        }
        var nstr = '';
        for (var i = 0; i < str.length; i++) {
          nstr += '\\' + str.charCodeAt(i).toString(16);
        }
        return nstr;
      };


      _.getReferrer = function(referrer) {
        var referrer = referrer || document.referrer;
        if (typeof referrer !== 'string') {
          return '取值异常_referrer异常_' + String(referrer);
        }
        if (referrer.indexOf('https://www.baidu.com/') === 0) {
          referrer = referrer.split('?')[0];
        }
        referrer = referrer.slice(0, webinsight.para.max_referrer_string_length);
        return typeof referrer === 'string' ? referrer : '';
      };

      _.getKeywordFromReferrer = function(referrerUrl) {
        referrerUrl = referrerUrl || document.referrer;
        var search_keyword = webinsight.para.source_type.keyword;
        if (document && typeof referrerUrl === 'string') {
          if (referrerUrl.indexOf('http') === 0) {
            var searchEngine = _.getReferSearchEngine(referrerUrl);
            var query = _.getQueryParamsFromUrl(referrerUrl);
            if (_.isEmptyObject(query)) {
              return '未取到值';
            }
            var temp = null;
            for (var i in search_keyword) {
              if (searchEngine === i) {
                if (typeof query === 'object') {
                  temp = search_keyword[i];
                  if (_.isArray(temp)) {
                    for (var i = 0; i < temp.length; i++) {
                      var _value = query[temp[i]];
                      if (_value) {
                        return _value;
                      }
                    }
                  } else if (query[temp]) {
                    return query[temp];
                  }
                }
              }
            }
            return '未取到值';
          } else {
            if (referrerUrl === '') {
              return '未取到值_直接打开';
            } else {
              return '未取到值_非http的url';
            }
          }
        } else {
          return '取值异常_referrer异常_' + String(referrerUrl);
        }
      };

      _.getWxAdIdFromUrl = function(url) {
        var click_id = _.getQueryParam(url, 'gdt_vid');
        var hash_key = _.getQueryParam(url, 'hash_key');
        var callbacks = _.getQueryParam(url, 'callbacks');
        var obj = {
          click_id: '',
          hash_key: '',
          callbacks: ''
        };
        if (_.isString(click_id) && click_id.length) {
          obj.click_id = click_id.length == 16 || click_id.length == 18 ? click_id : '参数解析不合法';

          if (_.isString(hash_key) && hash_key.length) {
            obj.hash_key = hash_key;
          }
          if (_.isString(callbacks) && callbacks.length) {
            obj.callbacks = callbacks;
          }
        }

        return obj;
      };

      _.getReferSearchEngine = function(referrerUrl) {
        var hostname = _.getHostname(referrerUrl);
        if (!hostname || hostname === 'hostname解析异常') {
          return '';
        }
        var search_keyword = webinsight.para.source_type.keyword;
        var searchEngineUrls = {
          baidu: [/^.*\.baidu\.com$/],
          bing: [/^.*\.bing\.com$/],
          google: [/^www\.google\.com$/, /^www\.google\.com\.[a-z]{2}$/, /^www\.google\.[a-z]{2}$/],
          sm: [/^m\.sm\.cn$/],
          so: [/^.+\.so\.com$/],
          sogou: [/^.*\.sogou\.com$/],
          yahoo: [/^.*\.yahoo\.com$/]
        };
        for (var prop in searchEngineUrls) {
          var urls = searchEngineUrls[prop];
          for (var i = 0, len = urls.length; i < len; i++) {
            if (urls[i].test(hostname)) {
              return prop;
            }
          }
        }
        return '未知搜索引擎';
      };

      _.getSourceFromReferrer = function() {
        function getMatchStrFromArr(arr, str) {
          for (var i = 0; i < arr.length; i++) {
            if (str.split('?')[0].indexOf(arr[i]) !== -1) {
              return true;
            }
          }
        }

        var utm_reg = '(' + webinsight.para.source_type.utm.join('|') + ')\\=[^&]+';
        var search_engine = webinsight.para.source_type.search;
        var social_engine = webinsight.para.source_type.social;

        var referrer = document.referrer || '';
        var url = _.info.pageProp.url;
        if (url) {
          var utm_match = url.match(new RegExp(utm_reg));
          if (utm_match && utm_match[0]) {
            return '付费广告流量';
          } else if (getMatchStrFromArr(search_engine, referrer)) {
            return '自然搜索流量';
          } else if (getMatchStrFromArr(social_engine, referrer)) {
            return '社交网站流量';
          } else if (referrer === '') {
            return '直接流量';
          } else {
            return '引荐流量';
          }
        } else {
          return '获取url异常';
        }
      };

      _.info = {
        initPage: function() {
          var referrer = _.getReferrer();
          var url = location.href;
          var url_domain = _.getCurrentDomain(url);

          this.pageProp = {
            referrer: referrer,
            referrer_host: referrer ? _.getHostname(referrer) : '',
            url: url,
            url_host: _.getHostname(url, 'url_host取值异常'),
            url_domain: url_domain
          };
        },
        pageProp: {},

        campaignParams: function() {
          var campaign_keywords = webinsight.source_channel_standard.split(' '),
            kw = '',
            params = {};
          if (_.isArray(webinsight.para.source_channel) && webinsight.para.source_channel.length > 0) {
            campaign_keywords = campaign_keywords.concat(webinsight.para.source_channel);
            campaign_keywords = _.unique(campaign_keywords);
          }
          _.each(campaign_keywords, function(kwkey) {
            kw = _.getQueryParam(location.href, kwkey);
            if (kw.length) {
              params[kwkey] = kw;
            }
          });

          return params;
        },
        campaignParamsStandard: function(prefix, prefix_add) {
          prefix = prefix || '';
          prefix_add = prefix_add || '';
          var utms = _.info.campaignParams();
          var $utms = {},
            otherUtms = {};
          _.each(utms, function(v, i, utms) {
            if ((' ' + webinsight.source_channel_standard + ' ').indexOf(' ' + i + ' ') !== -1) {
              $utms[prefix + i] = utms[i];
            } else {
              otherUtms[prefix_add + i] = utms[i];
            }
          });
          return {
            $utms: $utms,
            otherUtms: otherUtms
          };
        },
        properties: function() {
          return {
            $timezoneOffset: new Date().getTimezoneOffset(),
            $screenHeight: Number(screen.height) || 0,
            $screenWidth: Number(screen.width) || 0,
            $lib: 'js',
            $libVersion: String(webinsight.lib_version),
            $time: new Date().getTime()
          };
        },
        currentProps: {},
        register: function(obj) {
          _.extend(_.info.currentProps, obj);
        }
      };

      _.autoExeQueue = function() {
        var queue = {
          items: [],
          enqueue: function(val) {
            this.items.push(val);
            this.start();
          },
          dequeue: function() {
            return this.items.shift();
          },
          getCurrentItem: function() {
            return this.items[0];
          },
          isRun: false,
          start: function() {
            if (this.items.length > 0 && !this.isRun) {
              this.isRun = true;
              this.getCurrentItem().start();
            }
          },
          close: function() {
            this.dequeue();
            this.isRun = false;
            this.start();
          }
        };
        return queue;
      };

      _.trackLink = function(obj, event_name, event_prop) {
        obj = obj || {};
        var link = null;
        if (obj.ele) {
          link = obj.ele;
        }
        if (obj.event) {
          if (obj.target) {
            link = obj.target;
          } else {
            link = obj.event.target;
          }
        }

        event_prop = event_prop || {};
        if (!link || typeof link !== 'object') {
          return false;
        }
        if (!link.href || /^javascript/.test(link.href) || link.target || link.download || link.onclick) {
          webinsight.track(event_name, event_prop);
          return false;
        }

        function linkFunc(e) {
          e.stopPropagation();
          e.preventDefault();
          var hasCalled = false;

          function track_a_click() {
            if (!hasCalled) {
              hasCalled = true;
              location.href = link.href;
            }
          }
          setTimeout(track_a_click, 1000);
          webinsight.track(event_name, event_prop, track_a_click);
        }
        if (obj.event) {
          linkFunc(obj.event);
        }
        if (obj.ele) {
          _.addEvent(obj.ele, 'click', function(e) {
            linkFunc(e);
          });
        }
      };

      _.eventEmitter = function() {
        this._events = [];
        this.pendingEvents = [];
      };

      _.eventEmitter.prototype = {
        emit: function(type) {
          var args = [].slice.call(arguments, 1);

          _.each(this._events, function(val) {
            if (val.type !== type) {
              return;
            }
            val.callback.apply(val.context, args);
          });
        },
        on: function(event, callback, context) {
          if (typeof callback !== 'function') {
            return;
          }
          this._events.push({
            type: event,
            callback: callback,
            context: context || this
          });
        },
        tempAdd: function(event, data) {
          if (!data || !event) {
            return;
          }

          this.pendingEvents.push({
            type: event,
            data: data
          });
          this.pendingEvents.length > 20 ? this.pendingEvents.shift() : null;
        },
        isReady: function() {
          var that = this;
          this.tempAdd = this.emit;

          if (this.pendingEvents.length === 0) {
            return;
          }
          _.each(this.pendingEvents, function(val) {
            that.emit(val.type, val.data);
          });

          this.pendingEvents = [];
        }
      };

      _.rot13obfs = function(str, key) {
        str = String(str);
        key = typeof key === 'number' ? key : 13;
        var n = 126;

        var chars = str.split('');

        for (var i = 0, len = chars.length; i < len; i++) {
          var c = chars[i].charCodeAt(0);

          if (c < n) {
            chars[i] = String.fromCharCode((chars[i].charCodeAt(0) + key) % n);
          }
        }

        return chars.join('');
      };

      _.rot13defs = function(str) {
        var key = 13,
          n = 126,
          str = String(str);

        return _.rot13obfs(str, n - key);
      };

      _.urlSafeBase64 = (function() {
        var ENC = {
          '+': '-',
          '/': '_',
          '=': '.'
        };
        var DEC = {
          '-': '+',
          _: '/',
          '.': '='
        };

        var encode = function(base64) {
          return base64.replace(/[+/=]/g, function(m) {
            return ENC[m];
          });
        };

        var decode = function(safe) {
          return safe.replace(/[-_.]/g, function(m) {
            return DEC[m];
          });
        };

        var trim = function(string) {
          return string.replace(/[.=]{1,2}$/, '');
        };

        var isBase64 = function(string) {
          return /^[A-Za-z0-9+/]*[=]{0,2}$/.test(string);
        };

        var isUrlSafeBase64 = function(string) {
          return /^[A-Za-z0-9_-]*[.]{0,2}$/.test(string);
        };

        return {
          encode: encode,
          decode: decode,
          trim: trim,
          isBase64: isBase64,
          isUrlSafeBase64: isUrlSafeBase64
        };
      })();

      _.setCssStyle = function(css) {
        var style = document.createElement('style');
        style.type = 'text/css';
        try {
          style.appendChild(document.createTextNode(css));
        } catch (e) {
          style.styleSheet.cssText = css;
        }
        var head = document.getElementsByTagName('head')[0];
        var firstScript = document.getElementsByTagName('script')[0];
        if (head) {
          if (head.children.length) {
            head.insertBefore(style, head.children[0]);
          } else {
            head.appendChild(style);
          }
        } else {
          firstScript.parentNode.insertBefore(style, firstScript);
        }
      };

      _.isIOS = function() {
        return !!navigator.userAgent.match(/iPhone|iPad|iPod/i);
      };

      _.getIOSVersion = function() {
        try {
          var version = navigator.appVersion.match(/OS (\d+)[._](\d+)[._]?(\d+)?/);
          return version && version[1] ? Number.parseInt(version[1], 10) : '';
        } catch (e) {
          return '';
        }
      };

      _.getUA = function() {
        var Sys = {};
        var ua = navigator.userAgent.toLowerCase();
        var s;
        if ((s = ua.match(/opera.([\d.]+)/))) {
          Sys.opera = Number(s[1].split('.')[0]);
        } else if ((s = ua.match(/msie ([\d.]+)/))) {
          Sys.ie = Number(s[1].split('.')[0]);
        } else if ((s = ua.match(/edge.([\d.]+)/))) {
          Sys.edge = Number(s[1].split('.')[0]);
        } else if ((s = ua.match(/firefox\/([\d.]+)/))) {
          Sys.firefox = Number(s[1].split('.')[0]);
        } else if ((s = ua.match(/chrome\/([\d.]+)/))) {
          Sys.chrome = Number(s[1].split('.')[0]);
        } else if ((s = ua.match(/version\/([\d.]+).*safari/))) {
          Sys.safari = Number(s[1].match(/^\d*.\d*/));
        }
        return Sys;
      };

      _.jsonp = function(obj) {
        if (!(_.isObject(obj) && _.isString(obj.callbackName))) {
          webinsight.log('JSONP 请求缺少 callbackName');
          return false;
        }
        obj.success = _.isFunction(obj.success) ? obj.success : function() {};
        obj.error = _.isFunction(obj.error) ? obj.error : function() {};
        obj.data = obj.data || '';
        var script = document.createElement('script');
        var head = document.getElementsByTagName('head')[0];
        var timer = null;
        head.appendChild(script);
        if (_.isNumber(obj.timeout)) {
          timer = setTimeout(function() {
            obj.error('timeout');
            window[obj.callbackName] = function() {
              webinsight.log('call jsonp error');
            };
            timer = null;
            head.removeChild(script);
          }, obj.timeout);
        }
        window[obj.callbackName] = function(data) {
          obj.success(data);
          window[obj.callbackName] = function() {
            webinsight.log('call jsonp error');
          };
          clearTimeout(timer);
          timer = null;
          head.removeChild(script);
        };
        if (obj.url.indexOf('?') > -1) {
          obj.url += '&callbackName=' + obj.callbackName;
        } else {
          obj.url += '?callbackName=' + obj.callbackName;
        }
        if (_.isObject(obj.data)) {
          var arr = [];
          _.each(obj.data, function(value, key) {
            arr.push(key + '=' + value);
          });
          obj.data = arr.join('&');
          obj.url += '&' + obj.data;
        }
        script.onerror = function(err) {
          window[obj.callbackName] = function() {
            webinsight.log('call jsonp error');
          };
          clearTimeout(timer);
          timer = null;
          head.removeChild(script);
          obj.error(err);
        };
        script.src = obj.url;
      };

      _.secCheck = {
        isHttpUrl: function(str) {
          if (typeof str !== 'string') return false;
          var _regex = /^https?:\/\/.+/;
          if (_regex.test(str) === false) {
            webinsight.log('Invalid URL');
            return false;
          }
          return true;
        },
        removeScriptProtocol: function(str) {
          if (typeof str !== 'string') return '';
          var _regex = /^\s*javascript/i;
          while (_regex.test(str)) {
            str = str.replace(_regex, '');
          }
          return str;
        }
      };
    })();;


    webinsight.para_default = {
      preset_properties: {
        latest_utm: true,
        latest_traffic_source_type: true,
        latest_search_keyword: true,
        latest_referrer: true,
        latest_referrer_host: false,
        latest_landing_page: false,
        latest_wx_ad_click_id: undefined,
        url: true,
        title: true
      },
      encrypt_cookie: false,
      img_use_crossorigin: false,

      name: 'sa',
      max_referrer_string_length: 200,
      max_string_length: 500,
      cross_subdomain: true,
      show_log: true,
      is_debug: false,
      debug_mode: false,
      debug_mode_upload: false,

      session_time: 0,
      source_channel: [],
      auto_init: true,
      is_track_single_page: true,
      is_single_page: false,
      source_type: {},
      callback_timeout: 200,
      datasend_timeout: 3000,
      queue_timeout: 300,
      ignore_oom: true,
      app_js_bridge: true,
      url_is_decode: false
    };

    webinsight.addReferrerHost = function(data) {
      var defaultHost = '取值异常';
      if (_.isObject(data.properties)) {
        if (data.properties.$first_referrer) {
          data.properties.$first_referrer_host = _.getHostname(data.properties.$first_referrer, defaultHost);
        }
        if (data.type === 'track' || data.type === 'track_signup') {
          if ('$referrer' in data.properties) {
            data.properties.$referrer_host = data.properties.$referrer === '' ? '' : _.getHostname(data.properties.$referrer, defaultHost);
          }
          if (webinsight.para.preset_properties.latest_referrer && webinsight.para.preset_properties.latest_referrer_host) {
            data.properties.$latest_referrer_host = data.properties.$latest_referrer === '' ? '' : _.getHostname(data.properties.$latest_referrer, defaultHost);
          }
        }
      }
    };

    webinsight.addPropsHook = function(data) {
      if (webinsight.para.preset_properties && webinsight.para.preset_properties.url && (data.type === 'track' || data.type === 'track_signup') && typeof data.properties.$url === 'undefined') {
        data.properties.$url = _.isDecodeURI(webinsight.para.url_is_decode, window.location.href);
      }
      if (webinsight.para.preset_properties && webinsight.para.preset_properties.title && (data.type === 'track' || data.type === 'track_signup') && typeof data.properties.$title === 'undefined') {
        data.properties.$title = document.title;
      }
    };

    webinsight.initPara = function(para) {
      webinsight.para = {
        name: 'shujiantec',
        heatmap: {
          collect_tags:{
            div : true
          }
        }
      };

      webinsight.para = para || webinsight.para || {};

      var latestObj = {};
      if (_.isObject(webinsight.para.is_track_latest)) {
        for (var latestProp in webinsight.para.is_track_latest) {
          latestObj['latest_' + latestProp] = webinsight.para.is_track_latest[latestProp];
        }
      }
      webinsight.para.preset_properties = _.extend({}, webinsight.para_default.preset_properties, latestObj, webinsight.para.preset_properties || {});

      var i;
      for (i in webinsight.para_default) {
        if (webinsight.para[i] === void 0) {
          webinsight.para[i] = webinsight.para_default[i];
        }
      }
      if (typeof webinsight.para.server_url === 'string') {
        webinsight.para.server_url = _.trim(webinsight.para.server_url);
        if (webinsight.para.server_url) {
          if (webinsight.para.server_url.slice(0, 3) === '://') {
            webinsight.para.server_url = location.protocol.slice(0, -1) + webinsight.para.server_url;
          } else if (webinsight.para.server_url.slice(0, 2) === '//') {
            webinsight.para.server_url = location.protocol + webinsight.para.server_url;
          } else if (webinsight.para.server_url.slice(0, 4) !== 'http') {
            webinsight.para.server_url = '';
          }
        }
      }

      if (typeof webinsight.para.web_url === 'string' && (webinsight.para.web_url.slice(0, 3) === '://' || webinsight.para.web_url.slice(0, 2) === '//')) {
        if (webinsight.para.web_url.slice(0, 3) === '://') {
          webinsight.para.web_url = location.protocol.slice(0, -1) + webinsight.para.web_url;
        } else {
          webinsight.para.web_url = location.protocol + webinsight.para.web_url;
        }
      }

      webinsight.bridge.initPara();
      webinsight.bridge.initState();

      var utm_type = ['utm_source', 'utm_medium', 'utm_campaign', 'utm_content', 'utm_term'];
      var search_type = ['www.baidu.', 'm.baidu.', 'm.sm.cn', 'so.com', 'sogou.com', 'youdao.com', 'google.', 'yahoo.com/', 'bing.com/', 'ask.com/'];
      var social_type = ['weibo.com', 'renren.com', 'kaixin001.com', 'douban.com', 'qzone.qq.com', 'zhihu.com', 'tieba.baidu.com', 'weixin.qq.com'];
      var search_keyword = {
        baidu: ['wd', 'word', 'kw', 'keyword'],
        google: 'q',
        bing: 'q',
        yahoo: 'p',
        sogou: ['query', 'keyword'],
        so: 'q',
        sm: 'q'
      };

      if (typeof webinsight.para.source_type === 'object') {
        webinsight.para.source_type.utm = _.isArray(webinsight.para.source_type.utm) ? webinsight.para.source_type.utm.concat(utm_type) : utm_type;
        webinsight.para.source_type.search = _.isArray(webinsight.para.source_type.search) ? webinsight.para.source_type.search.concat(search_type) : search_type;
        webinsight.para.source_type.social = _.isArray(webinsight.para.source_type.social) ? webinsight.para.source_type.social.concat(social_type) : social_type;
        webinsight.para.source_type.keyword = _.isObject(webinsight.para.source_type.keyword) ? _.extend(search_keyword, webinsight.para.source_type.keyword) : search_keyword;
      }
      var collect_tags_default = {
        div: false
      };
      var ignore_tags_default = ['mark', '/mark', 'strong', 'b', 'em', 'i', 'u', 'abbr', 'ins', 'del', 's', 'sup'];
      if (_.isObject(webinsight.para.heatmap)) {
        webinsight.para.heatmap.clickmap = webinsight.para.heatmap.clickmap || 'default';
        webinsight.para.heatmap.scroll_notice_map = webinsight.para.heatmap.scroll_notice_map || 'default';
        webinsight.para.heatmap.scroll_delay_time = webinsight.para.heatmap.scroll_delay_time || 4000;
        webinsight.para.heatmap.scroll_event_duration = webinsight.para.heatmap.scroll_event_duration || 18000;
        webinsight.para.heatmap.renderRefreshTime = webinsight.para.heatmap.renderRefreshTime || 1000;
        webinsight.para.heatmap.loadTimeout = webinsight.para.heatmap.loadTimeout || 1000;
        var trackAttrs = _.isArray(webinsight.para.heatmap.track_attr) ?
          _.filter(webinsight.para.heatmap.track_attr, function(v) {
            return v && typeof v === 'string';
          }) :
          [];
        trackAttrs.push('data-click');
        webinsight.para.heatmap.track_attr = trackAttrs;

        if (_.isObject(webinsight.para.heatmap.collect_tags)) {
          if (webinsight.para.heatmap.collect_tags.div === true) {
            webinsight.para.heatmap.collect_tags.div = {
              ignore_tags: ignore_tags_default,
              max_level: 1
            };
          } else if (_.isObject(webinsight.para.heatmap.collect_tags.div)) {
            if (webinsight.para.heatmap.collect_tags.div.ignore_tags) {
              if (!_.isArray(webinsight.para.heatmap.collect_tags.div.ignore_tags)) {
                webinsight.log('ignore_tags 参数必须是数组格式');
                webinsight.para.heatmap.collect_tags.div.ignore_tags = ignore_tags_default;
              }
            } else {
              webinsight.para.heatmap.collect_tags.div.ignore_tags = ignore_tags_default;
            }
            if (webinsight.para.heatmap.collect_tags.div.max_level) {
              var supportedDivLevel = [1, 2, 3];
              if (_.indexOf(supportedDivLevel, webinsight.para.heatmap.collect_tags.div.max_level) === -1) {
                webinsight.para.heatmap.collect_tags.div.max_level = 1;
              }
            }
          } else {
            webinsight.para.heatmap.collect_tags.div = false;
          }
        } else {
          webinsight.para.heatmap.collect_tags = collect_tags_default;
        }
      }

      if (webinsight.para.noCache === true) {
        webinsight.para.noCache = '?' + new Date().getTime();
      } else {
        webinsight.para.noCache = '';
      }

      if (webinsight.para.callback_timeout > webinsight.para.datasend_timeout) {
        webinsight.para.datasend_timeout = webinsight.para.callback_timeout;
      }
      if (webinsight.para.callback_timeout > webinsight.para.queue_timeout) {
        webinsight.para.queue_timeout = webinsight.para.callback_timeout;
      }
      if (webinsight.para.queue_timeout > webinsight.para.datasend_timeout) {
        webinsight.para.datasend_timeout = webinsight.para.queue_timeout;
      }
    };

    webinsight.readyState = {
      state: 0,
      historyState: [],
      stateType: {
        1: '1-init未开始',
        2: '2-init开始',
        3: '3-store完成'
      },
      getState: function() {
        return this.historyState.join('\n');
      },
      setState: function(n) {
        if (String(n) in this.stateType) {
          this.state = n;
        }
        this.historyState.push(this.stateType[n]);
      }
    };

    webinsight.setPreConfig = function(sa) {
      webinsight.para = sa.para;
      webinsight._q = sa._q;
    };

    webinsight.setInitVar = function() {
      webinsight._t = webinsight._t || 1 * new Date();
      webinsight.lib_version = '1.0.0';
      webinsight.is_first_visitor = false;
      webinsight.source_channel_standard = 'utm_source utm_medium utm_campaign utm_content utm_term';
    };

    webinsight.log = function() {
      console.log(arguments[0]);
    };

    var commonWays = {
      setOnlineState: function(state) {
        if (state === true && _.isObject(webinsight.para.jsapp) && typeof webinsight.para.jsapp.getData === 'function') {
          webinsight.para.jsapp.isOnline = true;
          var arr = webinsight.para.jsapp.getData();
          if (_.isArray(arr) && arr.length > 0) {
            _.each(arr, function(str) {
              if (_.isJSONString(str)) {
                webinsight.sendState.pushSend(JSON.parse(str));
              }
            });
          }
        } else {
          webinsight.para.jsapp.isOnline = false;
        }
      },
      autoTrackIsUsed: false,
      isReady: function(callback) {
        callback();
      },
      getUtm: function() {
        return _.info.campaignParams();
      },
      getStayTime: function() {
        return (new Date() - webinsight._t) / 1000;
      },
      setInitReferrer: function() {
        var _referrer = _.getReferrer();
      },
      setSessionReferrer: function() {
        var _referrer = _.getReferrer();
        store.setSessionPropsOnce({
          _session_referrer: _referrer,
          _session_referrer_host: _.info.pageProp.referrer_host
        });
      },
      setDefaultAttr: function() {
        _.info.register({
          _current_url: location.href,
          _referrer: _.getReferrer(),
          _referring_host: _.info.pageProp.referrer_host
        });
      },
      trackAllHeatMap: function(target, props, callback) {
        if (typeof target === 'object' && target.tagName) {
          var tagName = target.tagName.toLowerCase();
          heatmap.start(null, target, tagName, props, callback);
        }
      },
      autoTrackSinglePage: function(para, callback) {
        if (this.autoTrackIsUsed) {
          var url = _.info.pageProp.url;
        } else {
          var url = _.info.pageProp.referrer;
        }
        para = _.isObject(para) ? para : {};

        para = _.isObject(para) ? para : {};

        function getUtm() {
          var utms = _.info.campaignParams();
          var $utms = {};
          _.each(utms, function(v, i, utms) {
            if ((' ' + webinsight.source_channel_standard + ' ').indexOf(' ' + i + ' ') !== -1) {
              $utms['$' + i] = utms[i];
            } else {
              $utms[i] = utms[i];
            }
          });
          return $utms;
        }

        if (para.not_set_profile) {
          delete para.not_set_profile;
        }

        function closure(p, c) {
          let referrer = _.isDecodeURI(webinsight.para.url_is_decode, url)
          let _url = _.isDecodeURI(webinsight.para.url_is_decode, location.href)
          webinsight.track(
            '$page_view',
            _.extend({
                $referrer: referrer,
                $referrerHost: _.getHostname(referrer),
                $url: _url,
                $urlHost: _.getHostname(_url),
                $urlPath: location.pathname,
                $title: document.title
              },
              p,
              getUtm()
            ),
            c
          );
          url = location.href;
        }
//        console.log("3227",para)
        closure(para, callback);
        this.autoTrackSinglePage = closure;
      },
      autoTrackWithoutProfile: function(para, callback) {
        para = _.isObject(para) ? para : {};
        this.autoTrack(_.extend(para, {
          not_set_profile: true
        }), callback);
      },
      autoTrack: function(para, callback) {
        para = _.isObject(para) ? para : {};

        var utms = _.info.campaignParams();
        var $utms = {};
        _.each(utms, function(v, i, utms) {
          if ((' ' + webinsight.source_channel_standard + ' ').indexOf(' ' + i + ' ') !== -1) {
            $utms['$' + i] = utms[i];
          } else {
            $utms[i] = utms[i];
          }
        });
        if (webinsight.is_first_visitor && !para.not_set_profile) {
          webinsight.is_first_visitor = false;
        }
        if (para.not_set_profile) {
          delete para.not_set_profile;
        }

        var current_page_url = location.href;

        if (webinsight.para.is_single_page) {
          _.addHashEvent(function() {
            var referrer = _.getReferrer(current_page_url);
            var _referrer = _.isDecodeURI(webinsight.para.url_is_decode, referrer)
            var url =  _.isDecodeURI(webinsight.para.url_is_decode, location.href);
            webinsight.track(
              '$page_view',
              _.extend({
                  $referrer: _referrer,
                  $referrerHost: _.getHostname(_referrer),
                  $url: url,
                  $urlHost: _.getHost(url),
                  $urlPath: location.pathname,
                  $title: document.title
                },
                $utms,
                para
              ),
              callback
            );
            current_page_url = location.href;
          });
        }

        let _referrer = _.isDecodeURI(webinsight.para.url_is_decode, _.getReferrer())
        webinsight.track(
          '$page_view',
          _.extend({
              $referrer: _referrer,
              $referrerHost: _.getHostname(_referrer),
              $url: _.isDecodeURI(webinsight.para.url_is_decode, location.href),
              $urlPath: location.pathname,
              $urlHost: location.hostname,
              $title: document.title
            },
            $utms,
            para
          ),
          callback
        );
        this.autoTrackIsUsed = true;
      },
      getAnonymousID: function() {
        if (_.isEmptyObject(webinsight.store._state)) {
          return '请先初始化SDK';
        } else {
          return webinsight.store._state._first_id || webinsight.store._state.first_id || webinsight.store._state._distinct_id || webinsight.store._state.distinct_id;
        }
      },
    };

    webinsight.quick = function() {
      var arg = Array.prototype.slice.call(arguments);
      var arg0 = arg[0];
      var arg1 = arg.slice(1);
      if (typeof arg0 === 'string' && commonWays[arg0]) {
        return commonWays[arg0].apply(commonWays, arg1);
      } else if (typeof arg0 === 'function') {
        arg0.apply(webinsight, arg1);
      } else {
        webinsight.log('quick方法中没有这个功能' + arg[0]);
      }
    };

    webinsight.track = function(e, p, c) {
      if (wiEvent.check({
          event: e,
          properties: p
        })) {
        wiEvent.send({
            event: e,
            properties: p
          },
          c
        );
      }
    };

    webinsight.trackLink = function(link, event_name, event_prop) {
      if (typeof link === 'object' && link.tagName) {
        _.trackLink({
          ele: link
        }, event_name, event_prop);
      } else if (typeof link === 'object' && link.target && link.event) {
        _.trackLink(link, event_name, event_prop);
      }
    };
    webinsight.trackLinks = function(link, event_name, event_prop) {
      var ele = link;
      event_prop = event_prop || {};
      if (!link || typeof link !== 'object') {
        return false;
      }
      if (!link.href || /^javascript/.test(link.href) || link.target) {
        return false;
      }
      _.addEvent(link, 'click', function(e) {
        e.preventDefault();
        var hasCalled = false;
        setTimeout(track_a_click, 1000);

        function track_a_click() {
          if (!hasCalled) {
            hasCalled = true;
            location.href = link.href;
          }
        }
        webinsight.track(event_name, event_prop, track_a_click);
      });
    };

    webinsight.registerPage = function(obj) {
      if (wiEvent.check({
          properties: obj
        })) {
        _.extend(_.info.currentProps, obj);
      } else {
        webinsight.log('register输入的参数有误');
      }
    };

    webinsight.clearAllRegister = function(arr) {
      store.clearAllProps(arr);
    };

    webinsight.clearPageRegister = function(arr) {
      if (_.isArray(arr) && arr.length > 0) {
        for (var i = 0; i < arr.length; i++) {
          if (_.isString(arr[i]) && arr[i] in _.info.currentProps) {
            delete _.info.currentProps[arr[i]];
          }
        }
      } else if (arr === true) {
        for (var i in _.info.currentProps) {
          delete _.info.currentProps[i];
        }
      }
    };

    webinsight.register = function(props) {
      if (wiEvent.check({
          properties: props
        })) {
        store.setProps(props);
      } else {
        webinsight.log('register输入的参数有误');
      }
    };

    webinsight.registerOnce = function(props) {
      if (wiEvent.check({
          properties: props
        })) {
        store.setPropsOnce(props);
      } else {
        webinsight.log('registerOnce输入的参数有误');
      }
    };

    webinsight.registerSession = function(props) {
      if (wiEvent.check({
          properties: props
        })) {
        store.setSessionProps(props);
      } else {
        webinsight.log('registerSession输入的参数有误');
      }
    };

    webinsight.registerSessionOnce = function(props) {
      if (wiEvent.check({
          properties: props
        })) {
        store.setSessionPropsOnce(props);
      } else {
        webinsight.log('registerSessionOnce输入的参数有误');
      }
    };

    webinsight.getPresetProperties = function() {
      function getUtm() {
        var utms = _.info.campaignParams();
        var $utms = {};
        _.each(utms, function(v, i, utms) {
          if ((' ' + webinsight.source_channel_standard + ' ').indexOf(' ' + i + ' ') !== -1) {
            $utms['$' + i] = utms[i];
          } else {
            $utms[i] = utms[i];
          }
        });
        return $utms;
      }

      var obj = {
        $referrer: _.isDecodeURI(webinsight.para.url_is_decode, _.info.pageProp.referrer) || '',
        $referrerHost: _.info.pageProp.referrer ? _.getHostname(_.info.pageProp.referrer) : '',
        $url: _.isDecodeURI(webinsight.para.url_is_decode, location.href),
        $urlPath: location.pathname,
        $title: document.title || '',
        _distinct_id: store.getDistinctId()
      };
      var result = _.extend({}, _.info.properties(), webinsight.store.getProps(), getUtm(), obj);
      if (webinsight.para.preset_properties.latest_referrer && webinsight.para.preset_properties.latest_referrer_host) {
        result.$latest_referrer_host = result.$latest_referrer === '' ? '' : _.getHostname(result.$latest_referrer);
      }
      return result;
    };

    webinsight.detectMode = function() {
      function trackMode() {
        webinsight.readyState.setState(3);
        webinsight.bridge.app_js_bridge_v1();
        _.info.initPage();

        if (webinsight.para.is_track_single_page) {
          // FIXME: 注入先触发一次
          var current_page_url = location.href;
          var referrer = _.getReferrer(current_page_url);
          let _referrer = _.isDecodeURI(webinsight.para.url_is_decode, referrer)
          webinsight.track(
              '$page_view',
              _.extend({
                    $referrer: _referrer,
                    $referrerHost: _.getHostname(_referrer),
                    $url: _.isDecodeURI(webinsight.para.url_is_decode, location.href),
                    $urlHost: location.hostname,
                    $urlPath: location.pathname,
                    $title: document.title
                  },
                  {},
                  {}
              ),
              function () {

              }
          );

          _.addSinglePageEvent(function(last_url) {
            var sendData = function(extraData) {
              extraData = extraData || {};
              if (last_url !== location.href) {
                _.info.pageProp.referrer = last_url;
                last_url = _.isDecodeURI(webinsight.para.url_is_decode, last_url);
                webinsight.quick('autoTrack', _.extend({
                  $url: _.isDecodeURI(webinsight.para.url_is_decode, location.href),
                  $referrer: last_url,
                    $referrerHost : _.getHostname(last_url)
                }, extraData));
              }
            };
            if (typeof webinsight.para.is_track_single_page === 'boolean') {
              sendData();
            } else if (typeof webinsight.para.is_track_single_page === 'function') {
              var returnValue = webinsight.para.is_track_single_page();
              if (_.isObject(returnValue)) {
                sendData(returnValue);
              } else if (returnValue === true) {
                sendData();
              }
            }
          });
        }

        webinsight.store.init();

        webinsight.readyState.setState(4);
        if (webinsight._q && _.isArray(webinsight._q) && webinsight._q.length > 0) {
          _.each(webinsight._q, function(content) {
            webinsight[content[0]].apply(webinsight, Array.prototype.slice.call(content[1]));
          });
        }

        if (_.isObject(webinsight.para.heatmap)) {
          heatmap.initHeatmap();
          heatmap.initScrollmap();
        }
      }

      if (webinsight.para.heatmap && webinsight.para.heatmap.collect_tags && _.isObject(webinsight.para.heatmap.collect_tags)) {
        _.each(webinsight.para.heatmap.collect_tags, function(val, key) {
          if ((key !== 'div') && val) {
            webinsight.heatmap.otherTags.push(key);
          }
        });
      }

      trackMode();

    };;

    var dataSend = {};

    dataSend.getInstance = function(data) {
      var sendType = this.getSendType(data);
      var obj = new this[sendType](data);
      var start = obj.start;
      obj.start = function() {
        if (_.isObject(webinsight.para.is_debug) && webinsight.para.is_debug.storage && webinsight.store.requests) {
          webinsight.store.requests.push({
            name: this.server_url,
            initiatorType: this.img ? 'img' : 'xmlhttprequest',
            entryType: 'resource',
            requestData: this.data
          });
        }
        var me = this;
        start.apply(this, arguments);
        setTimeout(function() {
          me.isEnd(true);
        }, webinsight.para.callback_timeout);
      };
      obj.end = function() {
        this.callback && this.callback();
        var self = this;
        setTimeout(function() {
          self.lastClear && self.lastClear();
        }, webinsight.para.datasend_timeout - webinsight.para.callback_timeout);
      };
      obj.isEnd = function(isDelay) {
        if (!this.received) {
          this.received = true;
          this.end();
          var self = this;
          if (isDelay) {
            if (webinsight.para.queue_timeout - webinsight.para.callback_timeout <= 0) {
              self.close();
            } else {
              setTimeout(function() {
                self.close();
              }, webinsight.para.queue_timeout - webinsight.para.callback_timeout);
            }
          } else {
            self.close();
          }
        }
      };

      return obj;
    };

    dataSend.getRealtimeInstance = function(data) {
      var sendType = this.getSendType(data);
      var obj = new this[sendType](data);
      obj.defaultData = data;
      var start = obj.start;
      obj.start = function() {
        var me = this;
        start.apply(this, arguments);
        setTimeout(function() {
          me.isEnd(true);
        }, webinsight.para.callback_timeout);
      };
      obj.end = function() {
        this.callback && this.callback();
        var self = this;
        setTimeout(function() {
          self.lastClear && self.lastClear();
        }, webinsight.para.datasend_timeout - webinsight.para.callback_timeout);
      };
      obj.isEnd = function(isDelay) {
        if (!this.received) {
          this.received = true;
          this.end();
        }
      };
      return obj;
    };

    dataSend.getSendType = function(data) {
      return 'image'
    };

    var sendState = {};
    webinsight.sendState = sendState;
    webinsight.events = new _.eventEmitter();
    sendState.queue = _.autoExeQueue();

    sendState.getSendCall = function(data, config, callback) {
      if (webinsight.readyState.state < 3) {
        webinsight.log('初始化没有完成');
        return false;
      }

      var originData = data;

      var requestData = {
        data: originData,
        config: config,
        callback: callback
      };

      webinsight.events.tempAdd('send', originData);
      webinsight.bridge.dataSend(requestData, this, callback);
      webinsight.log(originData);
    };

    sendState.prepareServerUrl = function(requestData) {
      if (typeof AppInsightJSBridge != 'undefined') {
        AppInsightJSBridge.logJsResult(JSON.stringify(requestData))
      }
    };

    sendState.sendCall = function(requestData, server_url, callback) {
      var data = {
        server_url: server_url,
        data: JSON.stringify(requestData.data),
        callback: callback,
        config: requestData.config
      };
      if (_.isObject(webinsight.para.jsapp) && !webinsight.para.jsapp.isOnline && typeof webinsight.para.jsapp.setData === 'function') {
        delete data.callback;
        data = JSON.stringify(data);
        webinsight.para.jsapp.setData(data);
      } else {
        this.pushSend(data);
      }
    };

    sendState.pushSend = function(data) {
      var instance = dataSend.getInstance(data);
      var me = this;
      instance.close = function() {
        me.queue.close();
      };
      this.queue.enqueue(instance);
    };


    var wiEvent = {};
    webinsight.wiEvent = wiEvent;

    wiEvent.checkOption = {
      regChecks: {
        regName: /^((?!^distinct_id$|^original_id$|^time$|^properties$|^id$|^first_id$|^second_id$|^users$|^events$|^event$|^user_id$|^date$|^datetime$)[a-zA-Z_$][a-zA-Z\d_$]{0,99})$/i
      },
      checkPropertiesKey: function(obj) {
        var me = this,
          flag = true;
        _.each(obj, function(content, key) {
          if (!me.regChecks.regName.test(key)) {
            flag = false;
          }
        });
        return flag;
      },
      check: function(a, b) {
        if (typeof this[a] === 'string') {
          return this[this[a]](b);
        } else if (_.isFunction(this[a])) {
          return this[a](b);
        }
      },
      str: function(s) {
        if (!_.isString(s)) {
          webinsight.log('请检查参数格式,必须是字符串');
          return true;
        } else {
          return true;
        }
      },
      properties: function(p) {
        _.strip_sa_properties(p);
        if (p) {
          if (_.isObject(p)) {
            if (this.checkPropertiesKey(p)) {
              return true;
            } else {
              webinsight.log('properties 里的自定义属性名需要是合法的变量名，不能以数字开头，且只包含：大小写字母、数字、下划线，自定义属性不能以 $ 开头');
              return true;
            }
          } else {
            webinsight.log('properties可以没有，但有的话必须是对象');
            return true;
          }
        } else {
          return true;
        }
      },
      propertiesMust: function(p) {
        _.strip_sa_properties(p);
        if (p === undefined || !_.isObject(p) || _.isEmptyObject(p)) {
          webinsight.log('properties必须是对象且有值');
          return true;
        } else {
          if (this.checkPropertiesKey(p)) {
            return true;
          } else {
            webinsight.log('properties 里的自定义属性名需要是合法的变量名，不能以数字开头，且只包含：大小写字母、数字、下划线，自定义属性不能以 $ 开头');
            return true;
          }
        }
      },
      event: function(s) {
        if (!_.isString(s) || !this['regChecks']['regName'].test(s)) {
          webinsight.log('请检查参数格式，eventName 必须是字符串，且需是合法的变量名，即不能以数字开头，且只包含：大小写字母、数字、下划线和 $,其中以 $ 开头的表明是系统的保留字段，自定义事件名请不要以 $ 开头');
          return true;
        } else {
          return true;
        }
      },
      test_id: 'str',
      group_id: 'str',
      distinct_id: function(id) {
        if (_.isString(id) && /^.{1,255}$/.test(id)) {
          return true;
        } else {
          webinsight.log('distinct_id必须是不能为空，且小于255位的字符串');
          return false;
        }
      }
    };

    wiEvent.check = function(p) {
      var flag = true;
      for (var i in p) {
        if (Object.prototype.hasOwnProperty.call(p, i) && !this.checkOption.check(i, p[i])) {
          return false;
        }
      }
      return flag;
    };

    wiEvent.send = function(p, callback) {
      var data = {
        distinct_id: store.getDistinctId(),
        properties: {}
      };

      if (_.isObject(p) && _.isObject(p.properties) && !_.isEmptyObject(p.properties) && p.properties.$lib_detail) {
        data.lib.$lib_detail = p.properties.$lib_detail;
        delete p.properties.$lib_detail;
      }
      _.extend(data, webinsight.store.getUnionId(), p);

      if (_.isObject(p.properties) && !_.isEmptyObject(p.properties)) {
        _.extend(data.properties, p.properties);
      }

      if (!p.type || p.type.slice(0, 7) !== 'profile') {

        data.properties = _.extend({}, _.info.properties(), store.getProps(), store.getSessionProps(), _.info.currentProps, data.properties);

        if (webinsight.para.preset_properties.latest_wx_ad_click_id === 'not_collect') {
          delete data.properties._latest_wx_ad_click_id;
          delete data.properties._latest_wx_ad_hash_key;
          delete data.properties._latest_wx_ad_callbacks;
        } else if (webinsight.para.preset_properties.latest_wx_ad_click_id && !_.isString(data.properties._latest_wx_ad_click_id)) {
          data.properties._latest_wx_ad_click_id = '取值异常';
          data.properties._latest_wx_ad_hash_key = '取值异常';
          data.properties._latest_wx_ad_callbacks = '取值异常';
        }
        if (_.isString(data.properties._latest_wx_ad_click_id)) {
          data.properties.$url = _.isDecodeURI(webinsight.para.url_is_decode, window.location.href);
        }
      }

      _.parseSuperProperties(data);

      _.filterReservedProperties(data.properties);
      _.searchObjDate(data);
      _.searchObjString(data);
      _.searchZZAppStyle(data);

      var data_config = _.searchConfigData(data.properties);

      webinsight.addReferrerHost(data);
      webinsight.addPropsHook(data);
      webinsight.sendState.getSendCall(data, data_config, callback);
    };

    var store = (webinsight.store = {
      requests: [],
      _sessionState: {},
      _state: {
        distinct_id: '',
        first_id: '',
        props: {}
      },
      getProps: function() {
        return this._state.props || {};
      },
      getSessionProps: function() {
        return this._sessionState;
      },
      getDistinctId: function() {
        return this._state._distinct_id || this._state.distinct_id;
      },
      getUnionId: function() {
        var obj = {};
        return obj;
      },
      getFirstId: function() {
        return this._state._first_id || this._state.first_id;
      },
      toState: function(ds) {
        var state = null;
        if (ds != null && _.isJSONString(ds)) {
          state = JSON.parse(ds);
          this._state = _.extend(state);
          if (state.distinct_id) {
            if (typeof state.props === 'object') {
              for (var key in state.props) {
                if (typeof state.props[key] === 'string') {
                  state.props[key] = state.props[key].slice(0, webinsight.para.max_referrer_string_length);
                }
              }
              this.save();
            }
          } else {
            this.set('distinct_id', _.UUID());
          }
        } else {
          this.set('distinct_id', _.UUID());
        }
      },
      initSessionState: function() {
        var ds = _.cookie.get('webinsightdatasession');
        var state = null;
        if (ds !== null && typeof(state = JSON.parse(ds)) === 'object') {
          this._sessionState = state || {};
        }
      },

      setOnce: function(a, b) {
        if (!(a in this._state)) {
          this.set(a, b);
        }
      },
      set: function(name, value) {
        this._state = this._state || {};
        var pre_id = this._state.distinct_id;
        this._state[name] = value;
        if (name === 'first_id') {
          delete this._state._first_id;
        } else if (name === 'distinct_id') {
          delete this._state._distinct_id;
        }
        this.save();
        if (name === 'distinct_id' && pre_id) {
          webinsight.events.tempAdd('changeDistinctId', value);
        }
      },
      change: function(name, value) {
        this._state['_' + name] = value;
      },
      setSessionProps: function(newp) {
        var props = this._sessionState;
        _.extend(props, newp);
        this.sessionSave(props);
      },
      setSessionPropsOnce: function(newp) {
        var props = this._sessionState;
        _.coverExtend(props, newp);
        this.sessionSave(props);
      },
      setProps: function(newp, isCover) {
        var props = {};
        if (!isCover) {
          props = _.extend(this._state.props || {}, newp);
        } else {
          props = newp;
        }
        for (var key in props) {
          if (typeof props[key] === 'string') {
            props[key] = props[key].slice(0, webinsight.para.max_referrer_string_length);
          }
        }
        this.set('props', props);
      },
      setPropsOnce: function(newp) {
        var props = this._state.props || {};
        _.coverExtend(props, newp);
        this.set('props', props);
      },
      clearAllProps: function(arr) {
        this._sessionState = {};
        if (_.isArray(arr) && arr.length > 0) {
          for (var i = 0; i < arr.length; i++) {
            if (_.isString(arr[i]) && arr[i].indexOf('latest_') === -1 && arr[i] in this._state.props) {
              delete this._state.props[arr[i]];
            }
          }
        } else {
          for (var i in this._state.props) {
            if (i.indexOf('latest_') !== 1) {
              delete this._state.props[i];
            }
          }
        }
        this.sessionSave({});
        this.save();
      },
      sessionSave: function(props) {
        this._sessionState = props;
        _.cookie.set('webinsightdatasession', JSON.stringify(this._sessionState), 0);
      },
      save: function() {
        var copyState = JSON.parse(JSON.stringify(this._state));
        delete copyState._first_id;
        delete copyState._distinct_id;

        var stateStr = JSON.stringify(copyState);
        if (webinsight.para.encrypt_cookie) {
          stateStr = _.cookie.encrypt(stateStr);
        }
        _.cookie.set(this.getCookieName(), stateStr, 73000, webinsight.para.cross_subdomain);
      },
      getCookieName: function() {
        var sub = '';
        if (webinsight.para.cross_subdomain === false) {
          try {
            sub = _.URL(location.href).hostname;
          } catch (e) {
            webinsight.log(e);
          }
          if (typeof sub === 'string' && sub !== '') {
            sub = 'webinsight_jssdk_' + sub.replace(/\./g, '_');
          } else {
            sub = 'webinsight_jssdk_root';
          }
        } else {
          sub = 'webinsightdatajssdkcross';
        }
        return sub;
      },
      init: function() {
        this.initSessionState();
        var uuid = _.UUID();
        var cross = _.cookie.get(this.getCookieName());
        cross = _.cookie.resolveValue(cross);
        if (cross === null) {
          webinsight.is_first_visitor = true;

          this.set('distinct_id', uuid);
        } else {
          if (!_.isJSONString(cross) || !JSON.parse(cross).distinct_id) {
            webinsight.is_first_visitor = true;
          }

          this.toState(cross);
        }
      }
    });

    webinsight.bridge = {
      is_verify_success: false,
      initPara: function() {
        var app_js_bridge_default = {
          is_send: true,
          white_list: [],
          is_mui: false
        };
        if (typeof webinsight.para.app_js_bridge === 'object') {
          webinsight.para.app_js_bridge = _.extend({}, app_js_bridge_default, webinsight.para.app_js_bridge);
        } else if (webinsight.para.use_app_track === true || webinsight.para.app_js_bridge === true || webinsight.para.use_app_track === 'only') {
          if (webinsight.para.use_app_track_is_send === false || webinsight.para.use_app_track === 'only') {
            app_js_bridge_default.is_send = false;
          }
          webinsight.para.app_js_bridge = _.extend({}, app_js_bridge_default);
        } else if (webinsight.para.use_app_track === 'mui') {
          app_js_bridge_default.is_mui = true;
          webinsight.para.app_js_bridge = _.extend({}, app_js_bridge_default);
        }
        if (webinsight.para.app_js_bridge.is_send === false) {
          webinsight.log('设置了 is_send:false,如果打通失败，数据将被丢弃！');
        }
      },
      initState: function() {
        if (_.isObject(webinsight.para.app_js_bridge) && !webinsight.para.app_js_bridge.is_mui) {
          if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.webinsightdataNativeTracker) {
            webinsight.bridge.is_verify_success = true;
          } else if (_.isObject(window.WebInsightData_APP_New_H5_Bridge) && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_get_server_url && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_track) {
            webinsight.bridge.is_verify_success = true;
          }
        }
      },
      initDefineBridgeInfo: function() {
        var resultObj = {
          touch_app_bridge: true,
          verify_success: false
        };

        if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.webinsightdataNativeTracker && window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage) {
          if (webinsight.bridge.is_verify_success) {
            resultObj.verify_success = 'success';
          } else {
            resultObj.verify_success = 'fail';
          }
        } else if (_.isObject(window.WebInsightData_APP_New_H5_Bridge) && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_get_server_url && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_track) {
          if (webinsight.bridge.is_verify_success) {
            resultObj.verify_success = 'success';
          } else {
            resultObj.verify_success = 'fail';
          }
        } else if (typeof WebInsightData_APP_JS_Bridge === 'object' && ((WebInsightData_APP_JS_Bridge.webinsightdata_verify && WebInsightData_APP_JS_Bridge.webinsightdata_visual_verify) || WebInsightData_APP_JS_Bridge.webinsightdata_track)) {
          if (WebInsightData_APP_JS_Bridge.webinsightdata_verify && WebInsightData_APP_JS_Bridge.webinsightdata_visual_verify) {
            if (WebInsightData_APP_JS_Bridge.webinsightdata_visual_verify(JSON.stringify({
                server_url: webinsight.para.server_url
              }))) {
              resultObj.verify_success = 'success';
            } else {
              resultObj.verify_success = 'fail';
            }
          } else {
            resultObj.verify_success = 'success';
          }
        } else {
          resultObj.touch_app_bridge = false;
        }

        return resultObj;
      },
      dataSend: function(requestData, that, callback) {
        var originData = requestData.data;
        console.log(requestData,that)
        that.prepareServerUrl(requestData);
      },
      app_js_bridge_v1: function() {
        var app_info = null;
        var todo = null;

        function setAppInfo(data) {
          app_info = data;
          if (_.isJSONString(app_info)) {
            app_info = JSON.parse(app_info);
          }
          if (todo) {
            todo(app_info);
            todo = null;
            app_info = null;
          }
        }

        function getAndroid() {
          if (typeof window.WebInsightData_APP_JS_Bridge === 'object' && window.WebInsightData_APP_JS_Bridge.webinsightdata_call_app) {
            app_info = WebInsightData_APP_JS_Bridge.webinsightdata_call_app();
            if (_.isJSONString(app_info)) {
              app_info = JSON.parse(app_info);
            }
          }
        }
        window.webinsightdata_app_js_bridge_call_js = function(data) {
          setAppInfo(data);
        };
        function calliOS() {

        }
        webinsight.getAppStatus = function(func) {
            app_info = null;
        };
      },
      supportAppCallJs: function() {
        window.webinsightdata_app_call_js = function(type, data) {
          if (type in window.webinsightdata_app_call_js.modules) {
            window.webinsightdata_app_call_js.modules[type](data);
          }
        };
        window.webinsightdata_app_call_js.modules = {};
      }
    };
    webinsight.JSBridge = function(obj) {
      this.list = {};
      this.type = obj.type;
      this.app_call_js = _.isFunction(obj.app_call_js) ? obj.app_call_js : function() {};
      this.init();
    };
    webinsight.JSBridge.prototype.init = function() {
      var that = this;
      if (!window.webinsightdata_app_call_js.modules[this.type]) {
        window.webinsightdata_app_call_js.modules[this.type] = function(data) {
          that.app_call_js(data);
        };
      }
    };
    webinsight.JSBridge.prototype.jsCallApp = function(data) {
      var appData = {
        callType: this.type,
        data: data
      };
      if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.webinsightdataNativeTracker && window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage) {
        window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage(JSON.stringify(appData));
      } else if (_.isObject(window.WebInsightData_APP_New_H5_Bridge) && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_js_call_app) {
        window.WebInsightData_APP_New_H5_Bridge.webinsightdata_js_call_app(JSON.stringify(appData));
      } else {
        webinsight.log('数据发往App失败，App没有暴露bridge');
        return false;
      }
    };
    webinsight.JSBridge.prototype.hasAppBridge = function() {
      if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.webinsightdataNativeTracker && window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage) {
        return 'ios';
      } else if (_.isObject(window.WebInsightData_APP_New_H5_Bridge) && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_js_call_app) {
        return 'android';
      } else {
        webinsight.log('App端bridge未暴露');
        return false;
      }
    };
    webinsight.JSBridge.prototype.requestToApp = function(obj) {
      var that = this;
      var data = _.isObject(obj.data) ? obj.data : {};
      if (!_.isFunction(obj.callback)) {
        obj.callback = function() {};
      }

      if (_.isObject(obj.timeout) && _.isNumber(obj.timeout.time)) {
        if (!_.isFunction(obj.timeout.callback)) {
          obj.timeout.callback = function() {};
        }
        obj.timer = setTimeout(function() {
          obj.timeout.callback();
          delete that.list[key];
        }, obj.timeout.time);
      }

      function getKey() {
        var d = new Date().getTime().toString(16);
        var m = String(_.getRandom()).replace('.', '').slice(1, 8);
        return d + '-' + m;
      }
      var key = getKey();
      this.list[key] = obj;
      var appData = {
        callType: this.type,
        data: data
      };
      appData.data.message_id = key;
      if (window.webkit && window.webkit.messageHandlers && window.webkit.messageHandlers.webinsightdataNativeTracker && window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage) {
        window.webkit.messageHandlers.webinsightdataNativeTracker.postMessage(JSON.stringify(appData));
      } else if (_.isObject(window.WebInsightData_APP_New_H5_Bridge) && window.WebInsightData_APP_New_H5_Bridge.webinsightdata_js_call_app) {
        window.WebInsightData_APP_New_H5_Bridge.webinsightdata_js_call_app(JSON.stringify(appData));
      } else {
        webinsight.log('数据发往App失败，App没有暴露bridge');
        return false;
      }
    };
    webinsight.JSBridge.prototype.double = function(data) {
      if (data.message_id) {
        var message = this.list[data.message_id];
        if (message) {
          if (message.timer) {
            clearTimeout(message.timer);
          }
          message.callback(data);
          delete this.list[data.message_id];
        }
      }
    };;


    var heatmap = (webinsight.heatmap = {
      otherTags: [],
      getTargetElement: function(element, e) {
        var that = this;
        var target = element;
        if (typeof target !== 'object') {
          return null;
        }
        if (typeof target.tagName !== 'string') {
          return null;
        }
        var tagName = target.tagName.toLowerCase();
        if (tagName.toLowerCase() === 'body' || tagName.toLowerCase() === 'html') {
          return null;
        }
        if (!target || !target.parentNode || !target.parentNode.children) {
          return null;
        }

        var parent_ele = target.parentNode;
        var hasA = that.hasElement(e.originalEvent || e);
        var trackAttrs = webinsight.para.heatmap.track_attr;
        var otherTags = that.otherTags;

        if (tagName === 'a' || tagName === 'button' || tagName === 'input' || tagName === 'textarea' || _.hasAttributes(target, trackAttrs)) {
          return target;
        } else if (_.indexOf(otherTags, tagName) > -1) {
          return target;
        } else if (parent_ele.tagName.toLowerCase() === 'button' || parent_ele.tagName.toLowerCase() === 'a' || _.hasAttributes(parent_ele, trackAttrs)) {
          return parent_ele;
        } else if (tagName === 'area' && parent_ele.tagName.toLowerCase() === 'map' && _.ry(parent_ele).prev().tagName && _.ry(parent_ele).prev().tagName.toLowerCase() === 'img') {
          return _.ry(parent_ele).prev();
        } else if (hasA) {
          return hasA;
        } else if (tagName === 'div' && webinsight.para.heatmap.collect_tags.div && that.isDivLevelValid(target)) {
          var max_level = webinsight.para.heatmap && webinsight.para.heatmap.collect_tags && webinsight.para.heatmap.collect_tags.div && webinsight.para.heatmap.collect_tags.div.max_level || 1;
          if (max_level > 1 || that.isCollectableDiv(target)) {
            return target;
          } else {
            return null;
          }
        } else if (that.isStyleTag(tagName) && webinsight.para.heatmap.collect_tags.div) {
          var parentTrackDiv = that.getCollectableParent(target);
          if (parentTrackDiv && that.isDivLevelValid(parentTrackDiv)) {
            return parentTrackDiv;
          }
        }
        return null;
      },
      getDivLevels: function(element, rootElement) {
        var path = heatmap.getElementPath(element, true, rootElement);
        var pathArr = path.split(' > ');
        var ans = 0;
        _.each(pathArr, function(tag) {
          if (tag === 'div') {
            ans++;
          }
        });
        return ans;
      },
      isDivLevelValid: function(element) {
        var max_level = webinsight.para.heatmap && webinsight.para.heatmap.collect_tags && webinsight.para.heatmap.collect_tags.div && webinsight.para.heatmap.collect_tags.div.max_level || 1;

        var allDiv = element.getElementsByTagName('div');
        for (var i = allDiv.length - 1; i >= 0; i--) {
          if (heatmap.getDivLevels(allDiv[i], element) > max_level) {
            return false;
          }
        }
        return true;
      },
      getElementPath: function(element, ignoreID, rootElement) {
        var names = [];
        while (element.parentNode) {
          if (element.id && !ignoreID && /^[A-Za-z][-A-Za-z0-9_:.]*$/.test(element.id)) {
            names.unshift(element.tagName.toLowerCase() + '#' + element.id);
            break;
          } else {
            if (rootElement && element === rootElement) {
              names.unshift(element.tagName.toLowerCase());
              break;
            } else if (element === document.body) {
              names.unshift('body');
              break;
            } else {
              names.unshift(element.tagName.toLowerCase());
            }
            element = element.parentNode;
          }
        }
        return names.join(' > ');
      },
      getClosestLi: function(element) {
        var getClosest = function(elem, selector) {
          for (; elem && elem !== document && elem.nodeType === 1; elem = elem.parentNode) {
            if (elem.tagName.toLowerCase() === selector) {
              return elem;
            }
          }
          return null;
        };
        return getClosest(element, 'li');
      },
      getElementPosition: function(element, elementPath, ignoreID) {
        var closestLi = webinsight.heatmap.getClosestLi(element);
        if (!closestLi) {
          return null;
        }
        var tag = element.tagName.toLowerCase();
        var sameTypeTags = closestLi.getElementsByTagName(tag);
        var sameTypeTagsLen = sameTypeTags.length;
        var arr = [];
        if (sameTypeTagsLen > 1) {
          for (var i = 0; i < sameTypeTagsLen; i++) {
            var elepath = webinsight.heatmap.getElementPath(sameTypeTags[i], ignoreID);
            if (elepath === elementPath) {
              arr.push(sameTypeTags[i]);
            }
          }
          if (arr.length > 1) {
            return _.indexOf(arr, element);
          }
        }

        function _getPosition(element) {
          var parentNode = element.parentNode;
          if (!parentNode) {
            return '';
          }
          var sameTypeSiblings = _.ry(element).getSameTypeSiblings();
          var typeLen = sameTypeSiblings.length;
          if (typeLen === 1) {
            return 0;
          }
          for (var i = 0, e = element; _.ry(e).previousElementSibling().ele; e = _.ry(e).previousElementSibling().ele, i++);
          return i;
        }
        return _getPosition(closestLi);
      },
      getDomIndex: function(el) {
        if (!el.parentNode) return -1;
        var i = 0;
        var nodeName = el.tagName;
        var list = el.parentNode.children;
        for (var n = 0; n < list.length; n++) {
          if (list[n].tagName === nodeName) {
            if (el === list[n]) {
              return i;
            } else {
              i++;
            }
          }
        }
        return -1;
      },
      selector: function(el) {
        var i = el.parentNode && 9 == el.parentNode.nodeType ? -1 : this.getDomIndex(el);
        if (el.getAttribute && el.getAttribute('id') && /^[A-Za-z][-A-Za-z0-9_:.]*$/.test(el.getAttribute('id')) && (!webinsight.para.heatmap || (webinsight.para.heatmap && webinsight.para.heatmap.element_selector !== 'not_use_id'))) {
          return '#' + el.getAttribute('id');
        } else {
          return el.tagName.toLowerCase() + (~i ? ':nth-of-type(' + (i + 1) + ')' : '');
        }
      },
      getDomSelector: function(el, arr) {
        if (!el || !el.parentNode || !el.parentNode.children) {
          return false;
        }
        arr = arr && arr.join ? arr : [];
        var name = el.nodeName.toLowerCase();
        if (!el || name === 'body' || 1 != el.nodeType) {
          arr.unshift('body');
          return arr.join(' > ');
        }
        arr.unshift(this.selector(el));
        if (el.getAttribute && el.getAttribute('id') && /^[A-Za-z][-A-Za-z0-9_:.]*$/.test(el.getAttribute('id')) && webinsight.para.heatmap && webinsight.para.heatmap.element_selector !== 'not_use_id') return arr.join(' > ');
        return this.getDomSelector(el.parentNode, arr);
      },
      na: function() {
        var a = document.documentElement.scrollLeft || window.pageXOffset;
        return parseInt(isNaN(a) ? 0 : a, 10);
      },
      i: function() {
        var a = 0;
        try {
          (a = (o.documentElement && o.documentElement.scrollTop) || m.pageYOffset), (a = isNaN(a) ? 0 : a);
        } catch (b) {
          a = 0;
        }
        return parseInt(a, 10);
      },
      getBrowserWidth: function() {
        var a = window.innerWidth || document.body.clientWidth;
        return isNaN(a) ? 0 : parseInt(a, 10);
      },
      getBrowserHeight: function() {
        var a = window.innerHeight || document.body.clientHeight;
        return isNaN(a) ? 0 : parseInt(a, 10);
      },
      getScrollWidth: function() {
        var a = parseInt(document.body.scrollWidth, 10);
        return isNaN(a) ? 0 : a;
      },
      W: function(a) {
        var b = parseInt(+a.clientX + +this.na(), 10);
        var a = parseInt(+a.clientY + +this.i(), 10);
        return {
          x: isNaN(b) ? 0 : b,
          y: isNaN(a) ? 0 : a
        };
      },
      start: function(ev, target, tagName, customProps, callback) {
        var userCustomProps = _.isObject(customProps) ? customProps : {};
        var userCallback = _.isFunction(callback) ? callback : _.isFunction(customProps) ? customProps : undefined;
        if (webinsight.para.heatmap && webinsight.para.heatmap.collect_element && !webinsight.para.heatmap.collect_element(target)) {
          return false;
        }

        var selector = this.getDomSelector(target);
        var prop = _.getEleInfo({
          target: target
        });

        prop.$elementSelector = selector ? selector : '';
        prop.$elementPath = webinsight.heatmap.getElementPath(target, webinsight.para.heatmap && webinsight.para.heatmap.element_selector === 'not_use_id');
        var element_position = webinsight.heatmap.getElementPosition(target, prop.$element_path, webinsight.para.heatmap && webinsight.para.heatmap.element_selector === 'not_use_id');
        if (_.isNumber(element_position)) {
          prop.$elementPosition = element_position;
        }
        if (webinsight.para.heatmap && webinsight.para.heatmap.custom_property) {
          var customP = webinsight.para.heatmap.custom_property(target);
          if (_.isObject(customP)) {
            prop = _.extend(prop, customP);
          }
        }
        prop = _.extend(prop, userCustomProps);
        if (tagName === 'a' && webinsight.para.heatmap && webinsight.para.heatmap.isTrackLink === true) {
          _.trackLink({
            event: ev,
            target: target
          }, '$web_click', prop);
        } else {
          webinsight.track('$web_click', prop, userCallback);
        }
      },
      hasElement: function(e) {
        var path = e._getPath ? e._getPath() : heatmap.getElementPath(e.target, true).split(' > ');
        if (_.isArray(path) && path.length > 0) {
          for (var i = 0; i < path.length; i++) {
            if (path[i] && path[i].tagName && path[i].tagName.toLowerCase() === 'a') {
              return path[i];
            }
          }
        }
        return false;
      },
      isStyleTag: function(tagname, isVisualMode) {
        var defaultTag = ['a', 'div', 'input', 'button', 'textarea'];
        var ignore_tags_default = ['mark', '/mark', 'strong', 'b', 'em', 'i', 'u', 'abbr', 'ins', 'del', 's', 'sup'];
        if (_.indexOf(defaultTag, tagname) > -1) {
          return false;
        }
        if (isVisualMode && (!webinsight.para.heatmap || !webinsight.para.heatmap.collect_tags || !webinsight.para.heatmap.collect_tags.div)) {
          return _.indexOf(ignore_tags_default, tagname) > -1;
        } else if (_.isObject(webinsight.para.heatmap) && _.isObject(webinsight.para.heatmap.collect_tags) && _.isObject(webinsight.para.heatmap.collect_tags.div) && _.indexOf(webinsight.para.heatmap.collect_tags.div.ignore_tags, tagname) > -1) {
          return true;
        }
        return false;
      },
      isCollectableDiv: function(target, isVisualMode) {
        try {
          if (target.children.length === 0) {
            return true;
          } else {
            for (var i = 0; i < target.children.length; i++) {
              if (target.children[i].nodeType !== 1) {
                continue;
              }
              var tag = target.children[i].tagName.toLowerCase();
              var max_level = webinsight.para && webinsight.para.heatmap && webinsight.para.heatmap.collect_tags && webinsight.para.heatmap.collect_tags.div && webinsight.para.heatmap.collect_tags.div.max_level;
              if ((tag === 'div' && max_level > 1) || this.isStyleTag(tag, isVisualMode)) {
                if (!this.isCollectableDiv(target.children[i], isVisualMode)) {
                  return false;
                }
              } else {
                return false;
              }
            }
            return true;
          }
        } catch (error) {
          webinsight.log(error);
        }
        return false;
      },
      getCollectableParent: function(target, isVisualMode) {
        try {
          var parent = target.parentNode;
          var parentName = parent ? parent.tagName.toLowerCase() : '';
          if (parentName === 'body') {
            return false;
          }
          var max_level = webinsight.para && webinsight.para.heatmap && webinsight.para.heatmap.collect_tags && webinsight.para.heatmap.collect_tags.div && webinsight.para.heatmap.collect_tags.div.max_level;
          if (parentName && parentName === 'div' && (max_level > 1 || this.isCollectableDiv(parent, isVisualMode))) {
            return parent;
          } else if (parent && this.isStyleTag(parentName, isVisualMode)) {
            return this.getCollectableParent(parent, isVisualMode);
          }
        } catch (error) {
          webinsight.log(error);
        }
        return false;
      },
      initScrollmap: function() {
        if (!_.isObject(webinsight.para.heatmap) || webinsight.para.heatmap.scroll_notice_map !== 'default') {
          return false;
        }

        var checkPage = function() {
          if (webinsight.para.scrollmap && _.isFunction(webinsight.para.scrollmap.collect_url) && !webinsight.para.scrollmap.collect_url()) {
            return false;
          }
          return true;
        };

        var interDelay = function(param) {
          var interDelay = {};
          interDelay.timeout = param.timeout || 1000;
          interDelay.func = param.func;
          interDelay.hasInit = false;
          interDelay.inter = null;
          interDelay.main = function(para, isClose) {
            this.func(para, isClose);
            this.inter = null;
          };
          interDelay.go = function(isNoDelay) {
            var me = this;
            var para = {};
            if (!this.inter) {
              para.$viewportPosition = (document.documentElement && document.documentElement.scrollTop) || window.pageYOffset || document.body.scrollTop || 0;
              para.$viewportPosition = Math.round(para.$viewport_position) || 0;
              para.$viewportHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight || 0;
              para.$viewportWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth || 0;
              if (isNoDelay) {preset_properties
                interDelay.main(para, true);
              } else {
                this.inter = setTimeout(function() {
                  interDelay.main(para);
                }, this.timeout);
              }
            }
          };
          return interDelay;
        };

        var delayTime = interDelay({
          timeout: 1000,
          func: function(para, isClose) {
            var offsetTop = (document.documentElement && document.documentElement.scrollTop) || window.pageYOffset || document.body.scrollTop || 0;
            var current_time = new Date();
            var delay_time = current_time - this.current_time;
            if ((delay_time > webinsight.para.heatmap.scroll_delay_time && offsetTop - para.$viewport_position !== 0) || isClose) {
              para.$url = _.isDecodeURI(webinsight.para.url_is_decode, location.href);
              para.$title = document.title;
              para.$urlPath = location.pathname;
              para.$urlHost = location.hostname;
              para.$eventDuration = Math.min(webinsight.para.heatmap.scroll_event_duration, parseInt(delay_time) / 1000);
              webinsight.track('$ae_stay', para);
            }
            this.current_time = current_time;
          }
        });

        delayTime.current_time = new Date();

        _.addEvent(window, 'scroll', function() {
          if (!checkPage()) {
            return false;
          }
          delayTime.go();
        });

        _.addEvent(window, 'unload', function() {
          if (!checkPage()) {
            return false;
          }
          delayTime.go('notime');
        });
      },
      initHeatmap: function() {
        var that = this;
        if (!_.isObject(webinsight.para.heatmap) || webinsight.para.heatmap.clickmap !== 'default') {
          return false;
        }

        if (_.isFunction(webinsight.para.heatmap.collect_url) && !webinsight.para.heatmap.collect_url()) {
          return false;
        }

        if (webinsight.para.heatmap.collect_elements === 'all') {
          webinsight.para.heatmap.collect_elements = 'all';
        } else {
          webinsight.para.heatmap.collect_elements = 'interact';
        }

        if (webinsight.para.heatmap.collect_elements === 'all') {
          _.addEvent(document, 'click', function(e) {
            var ev = e || window.event;
            if (!ev) {
              return false;
            }
            var target = ev.target || ev.srcElement;
            if (typeof target !== 'object') {
              return false;
            }
            if (typeof target.tagName !== 'string') {
              return false;
            }
            var tagName = target.tagName.toLowerCase();
            if (tagName === 'body' || tagName === 'html') {
              return false;
            }
            if (!target || !target.parentNode || !target.parentNode.children) {
              return false;
            }
            var parent_ele = target.parentNode.tagName.toLowerCase();
            if (parent_ele === 'a' || parent_ele === 'button') {
              that.start(ev, target.parentNode, parent_ele);
            } else {
              that.start(ev, target, tagName);
            }
          });
        } else {
          _.addEvent(document, 'click', function(e) {
            var ev = e || window.event;
            if (!ev) {
              return false;
            }
            var target = ev.target || ev.srcElement;
            var theTarget = webinsight.heatmap.getTargetElement(target, e);
            if (theTarget) {
              that.start(ev, theTarget, theTarget.tagName.toLowerCase());
            }
          });
        }
      }
    });;

    webinsight.init = function(para) {
//      console.log('webinsight.init')
      if (webinsight.readyState && webinsight.readyState.state && webinsight.readyState.state >= 2) {
        return false;
      }
//      console.log('webinsight detectMode')
      webinsight.setInitVar();
      webinsight.readyState.setState(2);
      webinsight.initPara(para);
      webinsight.bridge.supportAppCallJs();
      webinsight.detectMode();
    };

    var methods = ['getAppStatus', 'track', 'quick', 'register', 'registerPage', 'registerOnce', 'trackLink', 'clearAllRegister', 'clearPageRegister'];

    _.each(methods, function(method) {
      var oldFunc = webinsight[method];
      webinsight[method] = function() {
        if (webinsight.readyState.state < 3) {
          if (!_.isArray(webinsight._q)) {
            webinsight._q = [];
          }
          webinsight._q.push([method, arguments]);
          return false;
        }
        if (!webinsight.readyState.getState()) {
          try {
            console.error('请先初始化 WebInsight JS SDK');
          } catch (e) {
            webinsight.log(e);
          }
          return;
        }
        return oldFunc.apply(webinsight, arguments);
      };
    });;

    if (typeof window['webInsightDataAnalytic'] === 'string') {
      webinsight.setPreConfig(window[webInsightDataAnalytic]);
      window[webInsightDataAnalytic] = webinsight;
      window['webInsightDataAnalytic'] = webinsight;
      webinsight.init();
    } else if (typeof window['webInsightDataAnalytic'] === 'undefined') {
      window['webInsightDataAnalytic'] = webinsight;
      return webinsight;
    } else {
      return window['webInsightDataAnalytic'];
    }
  } catch (err) {
    if (typeof console === 'object' && console.log) {
      try {
        console.log(err);
      } catch (e) {
        webinsight.log(e);
      }
    }
  }
});



